import { SublimeConfig } from '../types/sublimeConfig';
import { ContractTransaction, Signer } from 'ethers';
import { CreditLineRequest } from '../types/Types';
import { BigNumberish } from '@ethersproject/bignumber';
import { TokenManager } from '../tokenManager';
import { Balance, Options as Overrides } from '../types/Types';
import { CreditLineEthUtils } from './utils/creditLineEthUtils';
/**
 * @class CreditLineApi
 */
export declare class CreditLineApi {
    /**
     * @description Instance of credit line contract
     */
    private creditLineContract;
    /**
     * @description Instance to fetch and token metadata
     */
    private tokenManager;
    /**
     * @description All sublime contract addresses
     */
    private config;
    /**
     * @description Signer Object
     */
    private signer;
    constructor(signer: Signer, config: SublimeConfig, tokenManager: TokenManager);
    /**
     * @description The returned object helps interact with polls that have ETH as collateral or borrow token
     * @returns Instance of Credit line Eth Utils.
     */
    CreditLineEthUtils(): CreditLineEthUtils;
    /**
     * @description Requests a new credit line to lender
     * @param request details of the credit line to request
     * @param options transaction options
     * @returns contract transaction
     */
    requestCreditLineToLender(request: CreditLineRequest, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Requests a new credit line to borrower
     * @param request: details of the credit line to request
     * @returns contract transaction
     */
    requestCreditLineToBorrower(request: CreditLineRequest, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Accpeting a credit line.
     * @param creditLine
     * @returns Contract Transaction
     */
    acceptCreditLine(creditLine: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Calculated the total accrued till now for a credit line
     * @param creditLineNumber
     * @returns debt in borrow tokens (easy readable)
     */
    calculateInterestAccrued(creditLineNumber: BigNumberish): Promise<Balance>;
    /**
     * @description Calcualte the total debt for a credit line till now
     * @param creditLineNumber
     * @returns debt in borrow tokens
     */
    calculateCurrentDebt(creditLineNumber: BigNumberish): Promise<Balance>;
    /**
     * @description Calculate the total amount that can be borrowed from the credit line
     * @param creditLineNumber
     * @returns borrowable amount in borrow tokens (easy readable)
     */
    calculateBorrowableAmount(creditLineNumber: BigNumberish): Promise<Balance>;
    /**
     * @description Cancel a credit line
     * @param id ID of the credit line
     * @param options
     */
    cancelCreditLine(id: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Calcualte the current collateral ratio of a credit line
     * @param creditLineNumber
     * @returns collateral ratio (easy readable)
     */
    calculateCurrentCollateralRatio(creditLineNumber: BigNumberish): Promise<Balance>;
    /**
     * @description Calculate the total collateral tokens held against a credit line
     * @param creditLineNumber
     * @returns collateral tokens number (easy readable)
     */
    calculateTotalCollateralTokens(creditLineNumber: BigNumberish): Promise<Balance>;
    /**
     * @description Calculate the amount the collateral that can be withdrawn from the credit line
     * @param creditLineNumber
     * @returns collateral tokens (easy readable)
     */
    withdrawableCollateral(creditLineNumber: BigNumberish): Promise<Balance>;
    /**
     * @description Withdraws the collateral from the credit line
     * @param creditLineNumber
     * @param amount
     * @param toSavingsAccount: boolean params that enables with withdrawn collateral to be. False by default
     * @param options transaction options
     * directly transferred from credit line to user's savings account
     * @returns Contract Transaction
     */
    withdrawCollateralFromCreditLine(creditLineNumber: BigNumberish, amount: string, toSavingsAccount?: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Withdraws all possible collateral from the credit line
     * @param creditLineNumber
     * @param toSavingsAccount
     * @param options
     * @returns
     */
    withdrawAllCollateralFromCreditLine(creditLineNumber: BigNumberish, toSavingsAccount?: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Deposit Collateral to a credit line
     * @param creditLineNumber
     * @param amount
     * @param fromSavingsAccount: when set to true, the collateral is transfered directly from savings account to credit line
     * @returns Contract Transaction
     */
    depositCollateral(creditLineNumber: BigNumberish, amount: string, fromSavingsAccount: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Borrow tokens from credit line
     * @param creditLineNumber
     * @param amount
     * @returns Contract Transaction
     */
    borrowFromCreditLine(creditLineNumber: BigNumberish, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Repay the amount to credit line
     * @param creditLineNumber
     * @param amount
     * @returns Contract Transaction
     */
    repayCreditLine(creditLineNumber: BigNumberish, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Close a credit line
     * @param creditLineNumber
     * @returns Contract Transaction
     */
    closeCreditLine(creditLineNumber: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Liquidate a credit line
     * @param creditLineNumber
     * @param toSavingsAccount: when true, the collateral liquidated will be sent to savings account directly
     * @returns Contract Transaction
     */
    liquidateCreditLine(creditLineNumber: BigNumberish, toSavingsAccount?: boolean, options?: Overrides): Promise<ContractTransaction>;
    updateBorrowLimitLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateIdealCollateralRatioLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateBorrowRateLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateProtocolFeeFraction(fee: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateLiquidatorRewardFraction(fee: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
}
