"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreditLineApi = void 0;
const CreditLine__factory_1 = require("../wrappers/factories/CreditLine__factory");
const Types_1 = require("../types/Types");
const bignumber_js_1 = require("bignumber.js");
const creditLineEthUtils_1 = require("./utils/creditLineEthUtils");
/**
 * @class CreditLineApi
 */
class CreditLineApi {
    constructor(signer, config, tokenManager) {
        this.creditLineContract = new CreditLine__factory_1.CreditLine__factory(signer).attach(config.creditLineContractAddress);
        this.config = config;
        this.tokenManager = tokenManager;
        this.signer = signer;
    }
    /**
     * @description The returned object helps interact with polls that have ETH as collateral or borrow token
     * @returns Instance of Credit line Eth Utils.
     */
    CreditLineEthUtils() {
        return new creditLineEthUtils_1.CreditLineEthUtils(this.signer, this.config, this.tokenManager);
    }
    /**
     * @description Requests a new credit line to lender
     * @param request details of the credit line to request
     * @param options transaction options
     * @returns contract transaction
     */
    requestCreditLineToLender(request, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(request.borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(request.borrowAsset);
            const borrowLimit = new bignumber_js_1.BigNumber(request.borrowLimit);
            if (borrowLimit.isNaN() || borrowLimit.isZero() || borrowLimit.isNegative()) {
                throw new Error('borrowLimit should be a valid number');
            }
            const borrowRate = new bignumber_js_1.BigNumber(request.borrowRate);
            if (borrowRate.isNaN() || borrowRate.isNegative()) {
                throw new Error('borrowRate should be a valid number');
            }
            const collateralRatio = new bignumber_js_1.BigNumber(request.collateralRatio);
            if (collateralRatio.isNaN() || collateralRatio.isNegative()) {
                throw new Error('collateralRatio should be a valid number');
            }
            let borrowStrategy;
            if (request.borrowAssetStrategy == Types_1.StrategyType.NoYield) {
                borrowStrategy = this.config.noStrategyAddress;
            }
            else if (request.borrowAssetStrategy == Types_1.StrategyType.CompoundYield) {
                borrowStrategy = this.config.compoundStrategyContractAddress;
            }
            else {
                throw new Error('Unsupported strategy');
            }
            let collateralStrategy;
            if (request.collateralAssetStrategy == Types_1.StrategyType.NoYield) {
                collateralStrategy = this.config.noStrategyAddress;
            }
            else if (request.collateralAssetStrategy == Types_1.StrategyType.CompoundYield) {
                collateralStrategy = this.config.compoundStrategyContractAddress;
            }
            else {
                throw new Error('Unsupported strategy');
            }
            return yield this.creditLineContract.request(request.address, borrowLimit.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal)).toFixed(0), borrowRate.multipliedBy(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(0), request.autoLiquidation, collateralRatio.multipliedBy(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(0), request.borrowAsset, borrowStrategy, request.collateralAsset, collateralStrategy, false, Object.assign({}, options));
        });
    }
    /**
     * @description Requests a new credit line to borrower
     * @param request: details of the credit line to request
     * @returns contract transaction
     */
    requestCreditLineToBorrower(request, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(request.borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(request.borrowAsset);
            const borrowLimit = new bignumber_js_1.BigNumber(request.borrowLimit);
            if (borrowLimit.isNaN() || borrowLimit.isZero() || borrowLimit.isNegative()) {
                throw new Error('borrowLimit should be a valid number');
            }
            const borrowRate = new bignumber_js_1.BigNumber(request.borrowRate);
            if (borrowRate.isNaN() || borrowRate.isNegative()) {
                throw new Error('borrowRate should be a valid number');
            }
            const collateralRatio = new bignumber_js_1.BigNumber(request.collateralRatio);
            if (collateralRatio.isNaN() || collateralRatio.isNegative()) {
                throw new Error('collateralRatio should be a valid number');
            }
            let borrowStrategy;
            if (request.borrowAssetStrategy == Types_1.StrategyType.NoYield) {
                borrowStrategy = this.config.noStrategyAddress;
            }
            else if (request.borrowAssetStrategy == Types_1.StrategyType.CompoundYield) {
                borrowStrategy = this.config.compoundStrategyContractAddress;
            }
            else {
                throw new Error('Unsupported strategy');
            }
            let collateralStrategy;
            if (request.collateralAssetStrategy == Types_1.StrategyType.NoYield) {
                collateralStrategy = this.config.noStrategyAddress;
            }
            else if (request.collateralAssetStrategy == Types_1.StrategyType.CompoundYield) {
                collateralStrategy = this.config.compoundStrategyContractAddress;
            }
            else {
                throw new Error('Unsupported strategy');
            }
            return yield this.creditLineContract.request(request.address, borrowLimit.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal)).toFixed(0), borrowRate.multipliedBy(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(0), request.autoLiquidation, collateralRatio.multipliedBy(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(0), request.borrowAsset, borrowStrategy, request.collateralAsset, collateralStrategy, true, Object.assign({}, options));
        });
    }
    /**
     * @description Accpeting a credit line.
     * @param creditLine
     * @returns Contract Transaction
     */
    acceptCreditLine(creditLine, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.creditLineContract.accept(creditLine, Object.assign({}, options));
        });
    }
    /**
     * @description Calculated the total accrued till now for a credit line
     * @param creditLineNumber
     * @returns debt in borrow tokens (easy readable)
     */
    calculateInterestAccrued(creditLineNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            let _value = '0';
            try {
                _value = yield (yield this.creditLineContract.calculateInterestAccrued(creditLineNumber)).toString();
            }
            catch (ex) { }
            const borrowAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            return { value: _value.toString(), decimals: borrowDecimal };
        });
    }
    /**
     * @description Calcualte the total debt for a credit line till now
     * @param creditLineNumber
     * @returns debt in borrow tokens
     */
    calculateCurrentDebt(creditLineNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            let _value = '0';
            try {
                _value = yield (yield this.creditLineContract.calculateCurrentDebt(creditLineNumber)).toString();
            }
            catch (ex) { }
            const borrowAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const decimals = this.tokenManager.getTokenDecimals(borrowAsset);
            return { value: _value.toString(), decimals };
        });
    }
    /**
     * @description Calculate the total amount that can be borrowed from the credit line
     * @param creditLineNumber
     * @returns borrowable amount in borrow tokens (easy readable)
     */
    calculateBorrowableAmount(creditLineNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            let _value = '0';
            try {
                _value = yield (yield this.creditLineContract.callStatic.calculateBorrowableAmount(creditLineNumber)).toString();
            }
            catch (ex) { }
            const borrowAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            return { value: _value.toString(), decimals: borrowDecimal };
        });
    }
    /**
     * @description Cancel a credit line
     * @param id ID of the credit line
     * @param options
     */
    cancelCreditLine(id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.cancel(id, Object.assign({}, options));
        });
    }
    /**
     * @description Calcualte the current collateral ratio of a credit line
     * @param creditLineNumber
     * @returns collateral ratio (easy readable)
     */
    calculateCurrentCollateralRatio(creditLineNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const [_value] = yield this.creditLineContract.callStatic.calculateCurrentCollateralRatio(creditLineNumber);
            return { value: _value.toString(), decimals: 18 };
        });
    }
    /**
     * @description Calculate the total collateral tokens held against a credit line
     * @param creditLineNumber
     * @returns collateral tokens number (easy readable)
     */
    calculateTotalCollateralTokens(creditLineNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const _value = yield this.creditLineContract.callStatic.calculateTotalCollateralTokens(creditLineNumber);
            const collateralAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).collateralAsset;
            yield this.tokenManager.updateTokenDecimals(collateralAsset);
            const collateralDecimal = this.tokenManager.getTokenDecimals(collateralAsset);
            return { value: _value.toString(), decimals: collateralDecimal };
        });
    }
    /**
     * @description Calculate the amount the collateral that can be withdrawn from the credit line
     * @param creditLineNumber
     * @returns collateral tokens (easy readable)
     */
    withdrawableCollateral(creditLineNumber) {
        return __awaiter(this, void 0, void 0, function* () {
            const collateralAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).collateralAsset;
            yield this.tokenManager.updateTokenDecimals(collateralAsset);
            const collateralDecimal = this.tokenManager.getTokenDecimals(collateralAsset);
            const result = yield this.creditLineContract.callStatic.withdrawableCollateral(creditLineNumber);
            return { value: result.toString(), decimals: collateralDecimal };
        });
    }
    /**
     * @description Withdraws the collateral from the credit line
     * @param creditLineNumber
     * @param amount
     * @param toSavingsAccount: boolean params that enables with withdrawn collateral to be. False by default
     * @param options transaction options
     * directly transferred from credit line to user's savings account
     * @returns Contract Transaction
     */
    withdrawCollateralFromCreditLine(creditLineNumber, amount, toSavingsAccount = false, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const collateralAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).collateralAsset;
            yield this.tokenManager.updateTokenDecimals(collateralAsset);
            const collateralDecimal = this.tokenManager.getTokenDecimals(collateralAsset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.creditLineContract['withdrawCollateral(uint256,uint256,bool)'](creditLineNumber, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(collateralDecimal)).toFixed(0), toSavingsAccount, Object.assign({}, options));
        });
    }
    /**
     * @description Withdraws all possible collateral from the credit line
     * @param creditLineNumber
     * @param toSavingsAccount
     * @param options
     * @returns
     */
    withdrawAllCollateralFromCreditLine(creditLineNumber, toSavingsAccount = false, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract['withdrawCollateral(uint256,bool)'](creditLineNumber, toSavingsAccount, Object.assign({}, options));
        });
    }
    /**
     * @description Deposit Collateral to a credit line
     * @param creditLineNumber
     * @param amount
     * @param fromSavingsAccount: when set to true, the collateral is transfered directly from savings account to credit line
     * @returns Contract Transaction
     */
    depositCollateral(creditLineNumber, amount, fromSavingsAccount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const collateralAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).collateralAsset;
            yield this.tokenManager.updateTokenDecimals(collateralAsset);
            const collateralDecimal = this.tokenManager.getTokenDecimals(collateralAsset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.creditLineContract.depositCollateral(creditLineNumber, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(collateralDecimal)).toFixed(0), fromSavingsAccount, Object.assign({}, options));
        });
    }
    /**
     * @description Borrow tokens from credit line
     * @param creditLineNumber
     * @param amount
     * @returns Contract Transaction
     */
    borrowFromCreditLine(creditLineNumber, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.creditLineContract.borrow(creditLineNumber, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Repay the amount to credit line
     * @param creditLineNumber
     * @param amount
     * @returns Contract Transaction
     */
    repayCreditLine(creditLineNumber, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowAsset = yield (yield this.creditLineContract.creditLineConstants(creditLineNumber)).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.creditLineContract.repay(creditLineNumber, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Close a credit line
     * @param creditLineNumber
     * @returns Contract Transaction
     */
    closeCreditLine(creditLineNumber, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.close(creditLineNumber, Object.assign({}, options));
        });
    }
    /**
     * @description Liquidate a credit line
     * @param creditLineNumber
     * @param toSavingsAccount: when true, the collateral liquidated will be sent to savings account directly
     * @returns Contract Transaction
     */
    liquidateCreditLine(creditLineNumber, toSavingsAccount = false, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.liquidate(creditLineNumber, toSavingsAccount, Object.assign({}, options));
        });
    }
    updateBorrowLimitLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.updateBorrowLimitLimits(min, max, Object.assign({}, options));
        });
    }
    updateIdealCollateralRatioLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.updateIdealCollateralRatioLimits(min, max, Object.assign({}, options));
        });
    }
    updateBorrowRateLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.updateBorrowRateLimits(min, max, Object.assign({}, options));
        });
    }
    updateProtocolFeeFraction(fee, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.updateProtocolFeeFraction(fee, Object.assign({}, options));
        });
    }
    updateLiquidatorRewardFraction(fee, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.creditLineContract.updateLiquidatorRewardFraction(fee, Object.assign({}, options));
        });
    }
}
exports.CreditLineApi = CreditLineApi;
//# sourceMappingURL=creditLines.js.map