import { SublimeConfig } from '../types/sublimeConfig';
import { StrategyType, CreditLineStatus, VerifierType, LenderWithdrawableAmount } from '../types/Types';
import { BigNumberish } from '@ethersproject/bignumber';
import { TokenManager } from '../tokenManager';
import { Balance, Options as Overrides } from '../types/Types';
import { ContractTransaction, Signer } from 'ethers';
/**
 * @class PooledCreditLines
 */
export declare class PooledCreditLineApi {
    /**
     * @description Instance of lender pool contract
     */
    private lenderPool;
    /**
     * @description Instance of pooled credit line contract
     */
    private pooledCreditLine;
    /**
     * @description Instance to fetch and update token metadata
     */
    private tokenManager;
    private signer;
    private yieldApi;
    private verifiactionApi;
    constructor(signer: Signer, config: SublimeConfig, tokenManager: TokenManager);
    /**
     * @description Start a pooled credit line
     * @param _id
     * @param options
     * @returns
     */
    start(_id: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Function to lend token to credit lines
     * @param _id
     * @param _amount
     * @param options
     * @returns
     */
    lend(_id: string, _amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Calculate the principle withdrawable
     * @param _id
     * @param lender
     * @returns
     */
    calculatePrincipleWithdrawable(_id: string, lender?: string): Promise<Balance>;
    /**
     * @description Withdraw Liquidity
     * @param _id
     * @param options
     * @returns
     */
    withdrawLiquidity(_id: string, options?: Overrides): Promise<ContractTransaction>;
    withdrawLiquidatedCollateral(id: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Withdraw Interest
     * @param _id
     * @param options
     * @returns
     */
    withdrawInterest(_id: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Liquidate credit line
     * @param _id
     * @param withdraw
     * @param options
     * @returns
     */
    liquidate(_id: string, withdraw: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description to-do
     * @param lenderAddress
     * @returns
     */
    getLenderInterest(_id: string, lenderAddress: string): Promise<Balance>;
    /**
     * @description get principle
     * @param _id
     * @returns Balance
     */
    getPrinciple(_id: string): Promise<Balance>;
    _getCreditLineStatus(_id: string): Promise<CreditLineStatus>;
    /**
     * @description Credit line status
     * @param _id
     * @returns
     */
    getCreditLineStatus(_id: string): Promise<CreditLineStatus>;
    /**
     * @description Calculate interest accured
     * @param _id
     * @returns
     */
    calculateInterestAccured(_id: string): Promise<Balance>;
    /**
     * @description calculate current debt
     * @param _id
     * @returns
     */
    calculateCurrentDebt(_id: string): Promise<Balance>;
    /**
     * @description calculate borrowable amount
     * @param _id
     * @returns
     */
    calculateBorrowableAmount(_id: string): Promise<Balance>;
    /**
     *
     * @param colRatio
     * @param durationInSeconds
     * @param lenderVerifier
     * @param defaultGracePeriodInSeconds
     * @param gracePenaltyRate
     * @param collectionPeriod
     * @param minBorrowAmount
     * @param _borrowLimit
     * @param _borrowRate
     * @param collateralAsset
     * @param borrowAssetStrategy
     * @param collateralAssetStrategy
     * @param borrowAsset
     * @param borrowerVerifier
     * @param areTokensTransferable
     * @param options
     * @returns
     */
    request(colRatio: string, durationInSeconds: string, lenderVerifier: VerifierType, defaultGracePeriodInSeconds: string, gracePenaltyRate: string, collectionPeriod: string, minBorrowAmount: string, _borrowLimit: string, _borrowRate: string, collateralAsset: string, borrowAssetStrategy: StrategyType, collateralAssetStrategy: StrategyType, borrowAsset: string, borrowerVerifier: VerifierType, areTokensTransferable: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Deposit Collateral
     * @param _id
     * @param _amount
     * @param _fromSavingsAccount
     * @param options
     * @returns
     */
    depositCollateral(_id: string, _amount: string, _fromSavingsAccount: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Borrow
     * @param _id
     * @param _amount
     * @param options
     * @returns
     */
    borrow(_id: string, _amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Repay
     * @param _id
     * @param _amount
     * @param options
     * @returns
     */
    repay(_id: string, _amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Close credit lines
     * @param _id
     * @param options
     * @returns
     */
    close(_id: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Cancels the credit lines
     * @param _id ID of the pooled credit line
     * @param options
     * @returns
     */
    cancel(_id: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description function withdraw partial collateral
     * @param _id
     * @param amount
     * @param toSavingsAccount
     * @param options
     */
    withdrawPartialCollateral(_id: string, _amount: string, toSavingsAccount: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description calculate current collateral ratio
     * @param _id
     * @returns
     */
    calculateCurrentCollateralRatio(_id: string): Promise<Balance>;
    /**
     * @description Calculate Total Collateral tokens
     * @param _id
     * @returns
     */
    calculateTokenCollateralTokens(_id: string): Promise<Balance>;
    /**
     * @description Withdraw Collateral
     * @param _id
     * @param _amount
     * @param toSavingsAccount
     * @param options
     * @returns
     */
    withdrawCollateral(_id: string, _amount: string, toSavingsAccount: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Withdraw All Collateral
     * @param _id
     * @param toSavingsAccount
     * @param options
     * @returns
     */
    withdrawAllCollateral(_id: string, toSavingsAccount: boolean, options?: Overrides): Promise<ContractTransaction>;
    calculateLenderWithdrawableAmount(_id: string, lender?: string): Promise<LenderWithdrawableAmount>;
    private calculateLenderInterest;
    updateBorrowLimitLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateIdealCollateralRatioLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateBorrowRateLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateCollectionPeriodLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateDurationLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateDefaultGracePeriodLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateGracePenaltyRateLimits(min: BigNumberish, max: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    updateProtocolFeeFraction(fee: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    getStatusAndUpdate(id: string): Promise<number>;
}
