"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PooledCreditLineApi = void 0;
const Types_1 = require("../types/Types");
const bignumber_js_1 = require("bignumber.js");
const PooledCreditLine__factory_1 = require("../wrappers/factories/PooledCreditLine__factory");
const LenderPool__factory_1 = require("../wrappers/factories/LenderPool__factory");
const IYield__factory_1 = require("../wrappers/factories/IYield__factory");
const yieldAndStrategy_1 = require("./yieldAndStrategy");
const verification_1 = require("./verification");
/**
 * @class PooledCreditLines
 */
class PooledCreditLineApi {
    constructor(signer, config, tokenManager) {
        this.tokenManager = tokenManager;
        this.signer = signer;
        this.lenderPool = new LenderPool__factory_1.LenderPool__factory(this.signer).attach(config.lenderPoolAddress);
        this.pooledCreditLine = new PooledCreditLine__factory_1.PooledCreditLine__factory(this.signer).attach(config.pooledCreditLineAddress);
        this.yieldApi = new yieldAndStrategy_1.YieldAndStrategyApi(signer, config, tokenManager);
        this.verifiactionApi = new verification_1.VerificationAPI(signer, config);
    }
    /**
     * @description Start a pooled credit line
     * @param _id
     * @param options
     * @returns
     */
    start(_id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.lenderPool.start(_id, Object.assign({}, options));
        });
    }
    /**
     * @description Function to lend token to credit lines
     * @param _id
     * @param _amount
     * @param options
     * @returns
     */
    lend(_id, _amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.lenderPool.pooledCLConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const amount = new bignumber_js_1.BigNumber(_amount);
            if (amount.isNaN() || amount.isZero() || amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.lenderPool.lend(_id, amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Calculate the principle withdrawable
     * @param _id
     * @param lender
     * @returns
     */
    calculatePrincipleWithdrawable(_id, lender) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!lender) {
                lender = yield this.signer.getAddress();
            }
            const token = yield (yield this.lenderPool.pooledCLConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const value = yield this.lenderPool.callStatic.calculatePrincipalWithdrawable(_id, lender);
            return { value: value.toString(), decimals };
        });
    }
    /**
     * @description Withdraw Liquidity
     * @param _id
     * @param options
     * @returns
     */
    withdrawLiquidity(_id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.lenderPool.withdrawLiquidity(_id, Object.assign({}, options));
        });
    }
    withdrawLiquidatedCollateral(id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.lenderPool.withdrawLiquidatedCollateral(id, Object.assign({}, options));
        });
    }
    /**
     * @description Withdraw Interest
     * @param _id
     * @param options
     * @returns
     */
    withdrawInterest(_id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.lenderPool.withdrawInterest(_id, Object.assign({}, options));
        });
    }
    /**
     * @description Liquidate credit line
     * @param _id
     * @param withdraw
     * @param options
     * @returns
     */
    liquidate(_id, withdraw, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.lenderPool.liquidate(_id, withdraw, Object.assign({}, options));
        });
    }
    /**
     * @description to-do
     * @param lenderAddress
     * @returns
     */
    getLenderInterest(_id, lenderAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowToken = yield (yield this.lenderPool.pooledCLConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(borrowToken);
            const value = yield this.lenderPool.callStatic.getLenderInterest(_id, lenderAddress);
            return { value: value.toString(), decimals: this.tokenManager.getTokenDecimals(borrowToken) };
        });
    }
    /**
     * @description get principle
     * @param _id
     * @returns Balance
     */
    getPrinciple(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.pooledCreditLine.pooledCreditLineConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const value = yield this.pooledCreditLine.getPrincipal(_id);
            return { value: value.toString(), decimals };
        });
    }
    _getCreditLineStatus(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            //   NOT_CREATED = 'NOT_CREATED',
            // REQUESTED = 'REQUESTED',
            // ACTIVE = 'ACTIVE',
            // CLOSED = 'CLOSED',
            // EXPIRED = 'EXPIRED',
            // LIQUIDATED = 'LIQUIDATED',
            // CANCELLED = 'CANCELLED',
            // START_CALLABLE = 'START_CALLABLE',
            // LIQUIDATE_CALLABLE = 'LIQUIDATE_CALLABLE',
            const pooledCreditLineVariables = yield this.pooledCreditLine.pooledCreditLineVariables(_id);
            const status = pooledCreditLineVariables.status;
            if (status == 0) {
                return Types_1.CreditLineStatus.NOT_CREATED;
            }
            else if (status == 1) {
                const pooledCLConstants = yield this.lenderPool.pooledCLConstants(_id);
                const amountLent = yield this.lenderPool.totalSupply(_id);
                const pooledCLConstantsOfPCL = yield this.pooledCreditLine.pooledCreditLineConstants(_id);
                const dateBigNumber = new bignumber_js_1.BigNumber(new Date().valueOf()).div(1000);
                if (dateBigNumber.gte(pooledCLConstantsOfPCL.startsAt.toString()) && amountLent.gte(pooledCLConstants.minBorrowAmount.toString())) {
                    return Types_1.CreditLineStatus.START_CALLABLE;
                }
                if (dateBigNumber.gte(pooledCLConstantsOfPCL.startsAt.toString()) && amountLent.lt(pooledCLConstants.minBorrowAmount.toString())) {
                    return Types_1.CreditLineStatus.CANCELLED;
                }
                return Types_1.CreditLineStatus.REQUESTED;
            }
            else if (status == 2) {
                const pooledCLConstants = yield this.pooledCreditLine.pooledCreditLineConstants(_id);
                const colRatio = yield this.pooledCreditLine.callStatic.calculateCurrentCollateralRatio(_id);
                if (pooledCLConstants.idealCollateralRatio.gt(colRatio)) {
                    return Types_1.CreditLineStatus.LIQUIDATE_CALLABLE;
                }
                const dateBigNumber = new bignumber_js_1.BigNumber(new Date().valueOf()).div(1000);
                const pooledCLVariables = yield this.pooledCreditLine.pooledCreditLineVariables(_id);
                if (dateBigNumber.gt(pooledCLConstants.endsAt.toString()) &&
                    dateBigNumber.lt(pooledCLConstants.defaultsAt.toString()) &&
                    pooledCLVariables.principal.gt(0)) {
                    return Types_1.CreditLineStatus.EXPIRED;
                }
                return Types_1.CreditLineStatus.ACTIVE;
            }
            else if (status == 3) {
                return Types_1.CreditLineStatus.CLOSED;
            }
            else if (status == 4) {
                const pooledCLConstants = yield this.pooledCreditLine.pooledCreditLineConstants(_id);
                const colRatio = yield this.pooledCreditLine.callStatic.calculateCurrentCollateralRatio(_id);
                if (pooledCLConstants.idealCollateralRatio.gt(colRatio)) {
                    return Types_1.CreditLineStatus.LIQUIDATE_CALLABLE;
                }
                return Types_1.CreditLineStatus.EXPIRED;
            }
            else if (status == 5) {
                return Types_1.CreditLineStatus.LIQUIDATED;
            }
            else if (status == 6) {
                return Types_1.CreditLineStatus.CANCELLED;
            }
            else if (status == 7) {
                return Types_1.CreditLineStatus.START_CALLABLE;
            }
            else if (status == 8) {
                return Types_1.CreditLineStatus.LIQUIDATE_CALLABLE;
            }
            else {
                throw new Error('Invalid State');
            }
        });
    }
    /**
     * @description Credit line status
     * @param _id
     * @returns
     */
    getCreditLineStatus(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._getCreditLineStatus(_id);
        });
    }
    /**
     * @description Calculate interest accured
     * @param _id
     * @returns
     */
    calculateInterestAccured(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.pooledCreditLine.pooledCreditLineConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const value = yield this.pooledCreditLine.calculateInterestAccrued(_id);
            return { value: value.toString(), decimals };
        });
    }
    /**
     * @description calculate current debt
     * @param _id
     * @returns
     */
    calculateCurrentDebt(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.pooledCreditLine.pooledCreditLineConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const value = yield this.pooledCreditLine.calculateCurrentDebt(_id);
            return { value: value.toString(), decimals };
        });
    }
    /**
     * @description calculate borrowable amount
     * @param _id
     * @returns
     */
    calculateBorrowableAmount(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.pooledCreditLine.pooledCreditLineConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const value = yield this.pooledCreditLine.callStatic.calculateBorrowableAmount(_id);
            return { value: value.toString(), decimals };
        });
    }
    /**
     *
     * @param colRatio
     * @param durationInSeconds
     * @param lenderVerifier
     * @param defaultGracePeriodInSeconds
     * @param gracePenaltyRate
     * @param collectionPeriod
     * @param minBorrowAmount
     * @param _borrowLimit
     * @param _borrowRate
     * @param collateralAsset
     * @param borrowAssetStrategy
     * @param collateralAssetStrategy
     * @param borrowAsset
     * @param borrowerVerifier
     * @param areTokensTransferable
     * @param options
     * @returns
     */
    request(colRatio, durationInSeconds, lenderVerifier, defaultGracePeriodInSeconds, gracePenaltyRate, collectionPeriod, minBorrowAmount, _borrowLimit, _borrowRate, collateralAsset, borrowAssetStrategy, collateralAssetStrategy, borrowAsset, borrowerVerifier, areTokensTransferable, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateAll(borrowAsset);
            const borrowTokenDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            const collateralRatio = new bignumber_js_1.BigNumber(colRatio);
            if (collateralRatio.isNaN() || collateralRatio.isNegative()) {
                throw new Error('collateralRatio should be a valid number');
            }
            const duration = new bignumber_js_1.BigNumber(durationInSeconds);
            if (duration.isNaN() || duration.isZero() || duration.isNegative()) {
                throw new Error('duration should be a valid number');
            }
            const defaultGracePeriod = new bignumber_js_1.BigNumber(defaultGracePeriodInSeconds);
            if (defaultGracePeriod.isNaN() || defaultGracePeriod.isZero() || defaultGracePeriod.isNegative()) {
                throw new Error('defaultGracePeriod should be a valid number');
            }
            const gpr = new bignumber_js_1.BigNumber(gracePenaltyRate);
            if (gpr.isNaN() || gpr.isNegative()) {
                throw new Error('gracePenaltyRate should be a valid number');
            }
            const _collectionPeriod = new bignumber_js_1.BigNumber(collectionPeriod);
            if (_collectionPeriod.isNaN() || _collectionPeriod.isZero() || _collectionPeriod.isNegative()) {
                throw new Error('_collectionPeriod should be a valid number');
            }
            const borrowLimit = new bignumber_js_1.BigNumber(_borrowLimit);
            if (borrowLimit.isNaN() || borrowLimit.isZero() || borrowLimit.isNegative()) {
                throw new Error('borrowLimit should be a valid number');
            }
            const borrowRate = new bignumber_js_1.BigNumber(_borrowRate);
            if (borrowRate.isNaN() || borrowRate.isNegative()) {
                throw new Error('borrowRate should be a valid number');
            }
            const _minBorrowAmount = new bignumber_js_1.BigNumber(minBorrowAmount);
            if (_minBorrowAmount.isNaN() || _minBorrowAmount.isZero() || _minBorrowAmount.isNegative()) {
                throw new Error('_minBorrowAmount should be a valid number');
            }
            const lenderVerifierAddress = this.verifiactionApi.getVerifierAddress(lenderVerifier);
            const collateralStrategyAddress = this.yieldApi.getStrategyAddress(collateralAssetStrategy);
            const borrowerStrategyAddress = this.yieldApi.getStrategyAddress(borrowAssetStrategy);
            const borrowerVerifierAddress = this.verifiactionApi.getVerifierAddress(borrowerVerifier);
            return this.pooledCreditLine.request({
                collateralRatio: collateralRatio.multipliedBy(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(0),
                duration: duration.toFixed(0),
                lenderVerifier: lenderVerifierAddress,
                defaultGracePeriod: defaultGracePeriod.toFixed(0),
                gracePenaltyRate: gpr.multipliedBy(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(0),
                collectionPeriod: _collectionPeriod.toFixed(0),
                minBorrowAmount: _minBorrowAmount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowTokenDecimal)).toFixed(0),
                borrowLimit: borrowLimit.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowTokenDecimal)).toFixed(0),
                borrowRate: borrowRate.multipliedBy(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(0),
                collateralAsset,
                borrowAssetStrategy: borrowerStrategyAddress,
                collateralAssetStrategy: collateralStrategyAddress,
                borrowAsset,
                borrowerVerifier: borrowerVerifierAddress,
                areTokensTransferable,
            }, Object.assign({}, options));
        });
    }
    /**
     * @description Deposit Collateral
     * @param _id
     * @param _amount
     * @param _fromSavingsAccount
     * @param options
     * @returns
     */
    depositCollateral(_id, _amount, _fromSavingsAccount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.pooledCreditLine.pooledCreditLineConstants(_id)).collateralAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const amount = new bignumber_js_1.BigNumber(_amount);
            if (amount.isNaN() || amount.isZero() || amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.pooledCreditLine.depositCollateral(_id, amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), _fromSavingsAccount, Object.assign({}, options));
        });
    }
    /**
     * @description Borrow
     * @param _id
     * @param _amount
     * @param options
     * @returns
     */
    borrow(_id, _amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.lenderPool.pooledCLConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const amount = new bignumber_js_1.BigNumber(_amount);
            if (amount.isNaN() || amount.isZero() || amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.pooledCreditLine.borrow(_id, amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Repay
     * @param _id
     * @param _amount
     * @param options
     * @returns
     */
    repay(_id, _amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.lenderPool.pooledCLConstants(_id)).borrowAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const amount = new bignumber_js_1.BigNumber(_amount);
            if (amount.isNaN() || amount.isZero() || amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.pooledCreditLine.repay(_id, amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Close credit lines
     * @param _id
     * @param options
     * @returns
     */
    close(_id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.close(_id, Object.assign({}, options));
        });
    }
    /**
     * @description Cancels the credit lines
     * @param _id ID of the pooled credit line
     * @param options
     * @returns
     */
    cancel(_id, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.cancelRequest(_id, Object.assign({}, options));
        });
    }
    /**
     * @description function withdraw partial collateral
     * @param _id
     * @param amount
     * @param toSavingsAccount
     * @param options
     */
    withdrawPartialCollateral(_id, _amount, toSavingsAccount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.lenderPool.pooledCLConstants(_id)).collateralAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const amount = new bignumber_js_1.BigNumber(_amount);
            if (amount.isNaN() || amount.isZero() || amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.pooledCreditLine['withdrawCollateral(uint256,uint256,bool)'](_id, amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), toSavingsAccount, Object.assign({}, options));
        });
    }
    /**
     * @description calculate current collateral ratio
     * @param _id
     * @returns
     */
    calculateCurrentCollateralRatio(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const value = yield this.pooledCreditLine.callStatic.calculateCurrentCollateralRatio(_id);
            return { value: value.toString(), decimals: 18 };
        });
    }
    /**
     * @description Calculate Total Collateral tokens
     * @param _id
     * @returns
     */
    calculateTokenCollateralTokens(_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const value = yield this.pooledCreditLine.callStatic.calculateTotalCollateralTokens(_id);
            const token = yield (yield this.pooledCreditLine.pooledCreditLineConstants(_id)).collateralAsset;
            yield this.tokenManager.updateAll(token);
            return { value: value.toString(), decimals: this.tokenManager.getTokenDecimals(token) };
        });
    }
    /**
     * @description Withdraw Collateral
     * @param _id
     * @param _amount
     * @param toSavingsAccount
     * @param options
     * @returns
     */
    withdrawCollateral(_id, _amount, toSavingsAccount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = yield (yield this.pooledCreditLine.pooledCreditLineConstants(_id)).collateralAsset;
            yield this.tokenManager.updateAll(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const amount = new bignumber_js_1.BigNumber(_amount);
            if (amount.isNaN() || amount.isZero() || amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.pooledCreditLine['withdrawCollateral(uint256,uint256,bool)'](_id, amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), toSavingsAccount, Object.assign({}, options));
        });
    }
    /**
     * @description Withdraw All Collateral
     * @param _id
     * @param toSavingsAccount
     * @param options
     * @returns
     */
    withdrawAllCollateral(_id, toSavingsAccount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine['withdrawCollateral(uint256,bool)'](_id, toSavingsAccount, Object.assign({}, options));
        });
    }
    calculateLenderWithdrawableAmount(_id, lender) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!lender) {
                lender = yield this.signer.getAddress();
            }
            const pclConstants = yield this.pooledCreditLine.pooledCreditLineConstants(_id);
            const pclVariables = yield this.pooledCreditLine.pooledCreditLineVariables(_id);
            const pclVariables2 = yield this.lenderPool.pooledCLVariables(_id);
            const lenderInfo = yield this.lenderPool.getLenderInfo(_id, lender);
            const lenderBalance = yield this.lenderPool.balanceOf(lender, _id);
            yield this.tokenManager.updateAll(pclConstants.borrowAsset);
            const decimals = this.tokenManager.getTokenDecimals(pclConstants.borrowAsset);
            const principal = yield this.lenderPool.callStatic.calculatePrincipalWithdrawable(_id, lender);
            const interest = yield this.calculateLenderInterest(_id, pclConstants.borrowLimit, pclVariables.principal, pclConstants.borrowAssetStrategy, pclConstants.borrowAsset, pclVariables2.sharesHeld, pclVariables2.borrowerInterestShares, lenderBalance, lenderInfo.borrowerInterestSharesWithdrawn, pclVariables2.borrowerInterestSharesWithdrawn, lenderInfo.yieldInterestWithdrawnShares);
            return {
                principal: { value: principal.toString(), decimals: decimals },
                interest: { value: interest.toString(), decimals: decimals },
            };
        });
    }
    calculateLenderInterest(_id, borrowLimit, principal, strategy, borrowAsset, sharesHeld, borrowerInterestShares, lenderBalance, lendersBorrowerInterestSharesWithdrawn, borrowerInterestSharesWithdrawn, yieldInterestWithdrawnShares) {
        return __awaiter(this, void 0, void 0, function* () {
            const amountNotBorrowed = new bignumber_js_1.BigNumber(borrowLimit.toString()).minus(principal.toString());
            const yieldContract = IYield__factory_1.IYield__factory.connect(strategy, this.signer);
            const notBorrowedInShares = yield yieldContract.callStatic.getSharesForTokens(amountNotBorrowed.toFixed(0), borrowAsset);
            const totalInterestInShares = new bignumber_js_1.BigNumber(sharesHeld.toString()).minus(notBorrowedInShares.toString());
            if (new bignumber_js_1.BigNumber(sharesHeld.toString()).eq(0)) {
                return new bignumber_js_1.BigNumber(0);
            }
            const borrowerInterestForLender = new bignumber_js_1.BigNumber(borrowerInterestShares.toString())
                .multipliedBy(lenderBalance.toString())
                .div(borrowLimit.toString())
                .minus(lendersBorrowerInterestSharesWithdrawn.toString());
            const borrowerInterestSharesAfterWithdrawal = new bignumber_js_1.BigNumber(borrowerInterestShares.toString()).minus(borrowerInterestSharesWithdrawn.toString());
            const totalYieldInterest = totalInterestInShares
                .minus(borrowerInterestSharesAfterWithdrawal)
                .plus(yieldInterestWithdrawnShares.toString());
            const yieldInterestForLender = totalYieldInterest
                .multipliedBy(lenderBalance.toString())
                .dividedBy(borrowLimit.toString())
                .minus(yieldInterestWithdrawnShares.toString());
            return yieldInterestForLender.plus(borrowerInterestForLender);
        });
    }
    updateBorrowLimitLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateBorrowLimitLimits(min, max, Object.assign({}, options));
        });
    }
    updateIdealCollateralRatioLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateIdealCollateralRatioLimits(min, max, Object.assign({}, options));
        });
    }
    updateBorrowRateLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateBorrowRateLimits(min, max, Object.assign({}, options));
        });
    }
    updateCollectionPeriodLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateCollectionPeriodLimits(min, max, Object.assign({}, options));
        });
    }
    updateDurationLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateDurationLimits(min, max, Object.assign({}, options));
        });
    }
    updateDefaultGracePeriodLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateDefaultGracePeriodLimits(min, max, Object.assign({}, options));
        });
    }
    updateGracePenaltyRateLimits(min, max, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateGracePenaltyRateLimits(min, max, Object.assign({}, options));
        });
    }
    updateProtocolFeeFraction(fee, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.updateProtocolFeeFraction(fee, Object.assign({}, options));
        });
    }
    getStatusAndUpdate(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.pooledCreditLine.callStatic.getStatusAndUpdate(id);
        });
    }
}
exports.PooledCreditLineApi = PooledCreditLineApi;
//# sourceMappingURL=pooledCreditLines.js.map