import { SublimeConfig } from '../types/sublimeConfig';
import { ContractTransaction, Signer } from 'ethers';
import { Asset, StrategyType, Options as Overrides, Balance } from '../types/Types';
import { SavingAccountsEthUtils } from './utils/savingsAccountEthUtils';
/**
 * @class SavingsAccountApi
 */
export declare class SavingsAccountApi {
    /**
     * @description Signer Object
     */
    private signer;
    /**
     * @description Instance of savings account contract
     */
    private savingsAccount;
    /**
     * @description All sublime addresses
     */
    private config;
    /**
     * @description Instance to fetch and update token metadata
     */
    private tokenManager;
    /**
     * @description Yield APIs
     */
    private yieldAndStrategyApi;
    constructor(signer: Signer, config: SublimeConfig, tokenManager: any);
    /**
     * @description The returned objet helps in making direct ETH deposits to the savings Account
     * @returns
     */
    SavingAccountsEthUtils(): SavingAccountsEthUtils;
    /**
     * @description Approve tokens in your wallet for depositing to a strategy
     * @param amount
     * @param asset
     * @param strategy
     * @returns
     */
    approveTokenForSavingsAccountDeposit(amount: string, asset: string, strategy: StrategyType, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description: Deposit amount to strategy
     * @param amount
     * @param asset
     * @param strategy
     * @param to
     * @returns Contract Transaction
     */
    deposit(amount: string, asset: string, strategy: StrategyType, to: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Switch amount from one strategy to another
     * @param currentStrategy
     * @param newStrategy
     * @param asset
     * @param amount
     * @returns Contract Transaction
     */
    switchStrategy(currentStrategy: StrategyType, newStrategy: StrategyType, asset: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Withdraw amount from a given strategy
     * @param amount
     * @param token
     * @param strategy
     * @param to
     * @param withdrawShares: if set to true, the "to" address will receive the shares token directly.
     * @returns Contract Transaction
     */
    withdraw(token: string, strategy: StrategyType, to: string, amount: string, withdrawShares: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Withdraw amount from another address i.e "from" address here
     * @param amount
     * @param token
     * @param strategy
     * @param from: address from which you wish to withdraw tokens
     * @param to
     * @param withdrawShares: if set to to true, the "to" address received shares directly
     * @returns Contract Transaction
     */
    withdrawFrom(amount: string, token: string, strategy: StrategyType, from: string, to: string, withdrawShares: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @param token
     * @param strategy
     * @param to
     * @param numberOfShares
     * @param receiveShares
     * @param options
     * @returns
     */
    withdrawShares(token: string, strategy: StrategyType, to: string, numberOfShares: string, receiveShares: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     *
     * @param token
     * @param strategy
     * @param from
     * @param to
     * @param numberOfShares
     * @param receiveShares
     * @param options
     * @returns
     */
    withdrawSharesFrom(token: string, strategy: StrategyType, from: string, to: string, numberOfShares: string, receiveShares: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Withdraw a particular tokens from all strategies
     * @param asset / token address
     * @returns Contract Transaction
     */
    withdrawAll(asset: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description withdraw all tokens from a strategy
     * @param asset asset to withdraw
     * @param strategy strategy from which the tokens should be withdrawn
     * @param options
     * @returns Contract Transaction
     */
    withdrawAllFromAStrategy(asset: string, strategy: StrategyType, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Approve tokens in the savings to be used to other address
     * @param amount
     * @param token
     * @param to : Address to which you want to approve
     * @returns Contract Transaction
     */
    approve(token: string, to: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    approveMax(token: string, to: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Increase the amount of tokens that can be used by "to" address
     * @param token
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    increaseAllowance(token: string, to: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Increase the amount of tokens that can be used by "to" address
     * @param token
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    decreaseAllowance(token: string, to: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     *
     * @param token Address of the token
     * @param from
     * @param to
     * @returns Allowance
     */
    getAllowance(token: string, from: string, to: string): Promise<Balance>;
    /**
     * @@deprecated
     * @param token
     * @param amount
     * @returns
     */
    approveTokenForCreditLines(token: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Transfer tokens from one address to another, but within the same strategy
     * @param amount
     * @param token
     * @param strategy
     * @param to
     * @returns Contract Transaction
     */
    transfer(token: string, strategy: StrategyType, to: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Transfer tokens from "from" address to another address
     * @param amount
     * @param token
     * @param strategy
     * @param from
     * @param to
     * @returns Contract Transaction
     */
    transferFrom(token: string, strategy: StrategyType, from: string, to: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Get specific token asset locked in savings account. For ex; if asset=dai_address,
     * then call returns the amount of DAI stored in the savings account for that user
     * @param user
     * @param asset
     * @returns easy read number in string
     */
    getTotalTokens(user: string, asset: string): Promise<Balance>;
    /**
     * @description Fet the specific liquidity shares in the savings account for a user.
     * @param user
     * @param asset
     * @param strategy
     * @returns easy read number in string
     */
    getShares(user: string, asset: string, strategy: StrategyType): Promise<Balance>;
    /**
     * @description Get the liquidity token for a given tokenb
     * @param asset
     * @param strategy
     * @returns Asset in detail
     */
    getLiquidityTokenOfAssetForStrategy(asset: string, strategy: StrategyType): Promise<Asset>;
    /**
     * @description Returns the contract address of a given strategy
     * @param strategy
     * @returns
     */
    private getStrategyAddress;
}
