"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SavingsAccountApi = void 0;
const SavingsAccount__factory_1 = require("../wrappers/factories/SavingsAccount__factory");
const bignumber_js_1 = require("bignumber.js");
const IYield__factory_1 = require("../wrappers/factories/IYield__factory");
const wrappers_1 = require("../wrappers");
const savingsAccountEthUtils_1 = require("./utils/savingsAccountEthUtils");
const yieldAndStrategy_1 = require("./yieldAndStrategy");
/**
 * @class SavingsAccountApi
 */
class SavingsAccountApi {
    constructor(signer, config, tokenManager) {
        this.signer = signer;
        this.config = config;
        this.savingsAccount = new SavingsAccount__factory_1.SavingsAccount__factory(signer).attach(config.savingsAccountContractAddress);
        this.yieldAndStrategyApi = new yieldAndStrategy_1.YieldAndStrategyApi(signer, config, tokenManager);
        this.tokenManager = tokenManager;
    }
    /**
     * @description The returned objet helps in making direct ETH deposits to the savings Account
     * @returns
     */
    SavingAccountsEthUtils() {
        return new savingsAccountEthUtils_1.SavingAccountsEthUtils(this.signer, this.config, this.tokenManager);
    }
    /**
     * @description Approve tokens in your wallet for depositing to a strategy
     * @param amount
     * @param asset
     * @param strategy
     * @returns
     */
    approveTokenForSavingsAccountDeposit(amount, asset, strategy, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            yield this.tokenManager.updateTokenDecimals(asset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(asset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            const tokenContract = new wrappers_1.ERC20__factory(this.signer).attach(asset);
            return tokenContract.approve(_strategyContractAddress, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description: Deposit amount to strategy
     * @param amount
     * @param asset
     * @param strategy
     * @param to
     * @returns Contract Transaction
     */
    deposit(amount, asset, strategy, to, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            yield this.tokenManager.updateTokenDecimals(asset);
            const assetDecimals = this.tokenManager.getTokenDecimals(asset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.deposit(asset, _strategyContractAddress, to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(assetDecimals)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Switch amount from one strategy to another
     * @param currentStrategy
     * @param newStrategy
     * @param asset
     * @param amount
     * @returns Contract Transaction
     */
    switchStrategy(currentStrategy, newStrategy, asset, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (currentStrategy === newStrategy) {
                throw new Error('Current Strategy and new strategy can not be same');
            }
            const _currentStrategyAddress = this.getStrategyAddress(currentStrategy);
            const _newStrategyAddress = this.getStrategyAddress(newStrategy);
            yield this.tokenManager.updateTokenDecimals(asset);
            const decimal = this.tokenManager.getTokenDecimals(asset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.switchStrategy(_currentStrategyAddress, _newStrategyAddress, asset, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Withdraw amount from a given strategy
     * @param amount
     * @param token
     * @param strategy
     * @param to
     * @param withdrawShares: if set to true, the "to" address will receive the shares token directly.
     * @returns Contract Transaction
     */
    withdraw(token, strategy, to, amount, withdrawShares, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            yield this.tokenManager.updateTokenDecimals(token);
            const decimal = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.withdraw(token, _strategyContractAddress, to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(0), withdrawShares, Object.assign({}, options));
        });
    }
    /**
     * @description Withdraw amount from another address i.e "from" address here
     * @param amount
     * @param token
     * @param strategy
     * @param from: address from which you wish to withdraw tokens
     * @param to
     * @param withdrawShares: if set to to true, the "to" address received shares directly
     * @returns Contract Transaction
     */
    withdrawFrom(amount, token, strategy, from, to, withdrawShares, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            yield this.tokenManager.updateTokenDecimals(token);
            const decimal = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('_amount should be a valid number');
            }
            return this.savingsAccount.withdrawFrom(token, _strategyContractAddress, from, to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(0), withdrawShares, Object.assign({}, options));
        });
    }
    /**
     * @param token
     * @param strategy
     * @param to
     * @param numberOfShares
     * @param receiveShares
     * @param options
     * @returns
     */
    withdrawShares(token, strategy, to, numberOfShares, receiveShares, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const strategyAddress = this.getStrategyAddress(strategy);
            return this.savingsAccount.withdrawShares(token, strategyAddress, to, numberOfShares, receiveShares, Object.assign({}, options));
        });
    }
    /**
     *
     * @param token
     * @param strategy
     * @param from
     * @param to
     * @param numberOfShares
     * @param receiveShares
     * @param options
     * @returns
     */
    withdrawSharesFrom(token, strategy, from, to, numberOfShares, receiveShares, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const strategyAddress = this.getStrategyAddress(strategy);
            return this.savingsAccount.withdrawSharesFrom(token, strategyAddress, from, to, numberOfShares, receiveShares, Object.assign({}, options));
        });
    }
    /**
     * @description Withdraw a particular tokens from all strategies
     * @param asset / token address
     * @returns Contract Transaction
     */
    withdrawAll(asset, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.savingsAccount['withdrawAll(address)'](asset, Object.assign({}, options));
        });
    }
    /**
     * @description withdraw all tokens from a strategy
     * @param asset asset to withdraw
     * @param strategy strategy from which the tokens should be withdrawn
     * @param options
     * @returns Contract Transaction
     */
    withdrawAllFromAStrategy(asset, strategy, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            return this.savingsAccount['withdrawAll(address,address)'](asset, _strategyContractAddress, Object.assign({}, options));
        });
    }
    /**
     * @description Approve tokens in the savings to be used to other address
     * @param amount
     * @param token
     * @param to : Address to which you want to approve
     * @returns Contract Transaction
     */
    approve(token, to, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.approve(token, to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), Object.assign({}, options));
        });
    }
    approveMax(token, to, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const max = new bignumber_js_1.BigNumber(2).pow(256).minus(1);
            return this.savingsAccount.approve(token, to, max.toFixed(), Object.assign({}, options));
        });
    }
    /**
     * @description Increase the amount of tokens that can be used by "to" address
     * @param token
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    increaseAllowance(token, to, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.increaseAllowance(_amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), token, to, Object.assign({}, options));
        });
    }
    /**
     * @description Increase the amount of tokens that can be used by "to" address
     * @param token
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    decreaseAllowance(token, to, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.decreaseAllowance(token, to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     *
     * @param token Address of the token
     * @param from
     * @param to
     * @returns Allowance
     */
    getAllowance(token, from, to) {
        return __awaiter(this, void 0, void 0, function* () {
            const allowance = yield this.savingsAccount.connect(this.signer).allowance(token, from, to);
            yield this.tokenManager.updateAll(token);
            return {
                value: allowance.toString(),
                decimals: this.tokenManager.getTokenDecimals(token),
            };
        });
    }
    /**
     * @@deprecated
     * @param token
     * @param amount
     * @returns
     */
    approveTokenForCreditLines(token, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.approve(token, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), this.config.creditLineContractAddress, Object.assign({}, options));
        });
    }
    /**
     * @description Transfer tokens from one address to another, but within the same strategy
     * @param amount
     * @param token
     * @param strategy
     * @param to
     * @returns Contract Transaction
     */
    transfer(token, strategy, to, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            yield this.tokenManager.updateTokenDecimals(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.transfer(token, _strategyContractAddress, to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Transfer tokens from "from" address to another address
     * @param amount
     * @param token
     * @param strategy
     * @param from
     * @param to
     * @returns Contract Transaction
     */
    transferFrom(token, strategy, from, to, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            yield this.tokenManager.updateTokenDecimals(token);
            const decimals = this.tokenManager.getTokenDecimals(token);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.savingsAccount.transferFrom(token, _strategyContractAddress, from, to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Get specific token asset locked in savings account. For ex; if asset=dai_address,
     * then call returns the amount of DAI stored in the savings account for that user
     * @param user
     * @param asset
     * @returns easy read number in string
     */
    getTotalTokens(user, asset) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(asset);
            const decimals = this.tokenManager.getTokenDecimals(asset);
            const getTotalTokens = yield (yield this.savingsAccount.callStatic.getTotalTokens(user, asset)).toString();
            return { value: getTotalTokens.toString(), decimals };
        });
    }
    /**
     * @description Fet the specific liquidity shares in the savings account for a user.
     * @param user
     * @param asset
     * @param strategy
     * @returns easy read number in string
     */
    getShares(user, asset, strategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            const yieldContract = IYield__factory_1.IYield__factory.connect(_strategyContractAddress, this.signer);
            const liquiditySharesAddress = yield yieldContract.liquidityToken(asset);
            yield this.tokenManager.updateTokenDecimals(liquiditySharesAddress);
            const decimal = this.tokenManager.getTokenDecimals(liquiditySharesAddress);
            const userShares = yield (yield this.savingsAccount.balanceInShares(user, asset, _strategyContractAddress)).toString();
            return { value: userShares.toString(), decimals: decimal };
        });
    }
    /**
     * @description Get the liquidity token for a given tokenb
     * @param asset
     * @param strategy
     * @returns Asset in detail
     */
    getLiquidityTokenOfAssetForStrategy(asset, strategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const _strategyContractAddress = this.getStrategyAddress(strategy);
            const yieldContract = IYield__factory_1.IYield__factory.connect(_strategyContractAddress, this.signer);
            const liquidityToken = yield yieldContract.liquidityToken(asset);
            yield this.tokenManager.updateAll(liquidityToken);
            return {
                address: liquidityToken,
                name: this.tokenManager.getTokenName(liquidityToken),
                pricePerAssetInUSD: (yield this.tokenManager.getPricePerAsset(liquidityToken)).toString(),
                logo: this.tokenManager.getLogo(liquidityToken),
                decimals: this.tokenManager.getTokenDecimals(liquidityToken),
            };
        });
    }
    /**
     * @description Returns the contract address of a given strategy
     * @param strategy
     * @returns
     */
    getStrategyAddress(strategy) {
        return this.yieldAndStrategyApi.getStrategyAddress(strategy);
    }
}
exports.SavingsAccountApi = SavingsAccountApi;
//# sourceMappingURL=savingsAccount.js.map