"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenApi = void 0;
const ERC20__factory_1 = require("../wrappers/factories/ERC20__factory");
const bignumber_js_1 = require("bignumber.js");
const wrappers_1 = require("../wrappers");
/**
 * @class TokenApi
 */
class TokenApi {
    /**
     * @description if tokenAddress is any invalid token address, the APIs of the class object will fail
     * @param signer
     * @param tokenAddress
     * @param tokenManager
     */
    constructor(signer, tokenAddress, tokenManager) {
        this.signer = signer;
        this.tokenContract = new ERC20__factory_1.ERC20__factory(signer).attach(tokenAddress);
        this.minterTokenContract = wrappers_1.IERC20Minter__factory.connect(tokenAddress, signer);
        this.tokenManager = tokenManager;
    }
    /**
     * @description Increase Allowance to a another address
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    IncreaseAllowance(to, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('shares should be a valid number');
            }
            yield this.tokenManager.updateTokenDecimals(this.tokenContract.address);
            const decimal = yield this.tokenManager.getTokenDecimals(this.tokenContract.address);
            return this.tokenContract
                .connect(this.signer)
                .increaseAllowance(to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Approve Allowance to a another address
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    approve(to, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('shares should be a valid number');
            }
            yield this.tokenManager.updateTokenDecimals(this.tokenContract.address);
            const decimal = yield this.tokenManager.getTokenDecimals(this.tokenContract.address);
            return this.tokenContract
                .connect(this.signer)
                .approve(to, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(0), Object.assign({}, options));
        });
    }
    /**
     * @description Returns the allowance of spender by owner
     * @param owner
     * @param spender
     * @param prettified, if true, the result will be in easy readable form
     * @returns number
     */
    allowance(owner, spender, prettified = false) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(this.tokenContract.address);
            const decimal = yield this.tokenManager.getTokenDecimals(this.tokenContract.address);
            const allowance = yield this.tokenContract.connect(this.signer).allowance(owner, spender);
            if (prettified) {
                return new bignumber_js_1.BigNumber(allowance.toString()).div(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(2);
            }
            else {
                return allowance.toString();
            }
        });
    }
    /**
     * @description return the current signers token balance.
     * @param prettified
     * @returns
     */
    getTokenBalance(prettified = true) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._getBalance(yield this.signer.getAddress(), prettified);
        });
    }
    /**
     * @description return any user token balance
     * @param user
     * @param prettified
     * @returns
     */
    getUserTokenBalance(user, prettified = true) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._getBalance(user, prettified);
        });
    }
    /**
     *
     * @param user address whose balance needs to be fetched
     * @param prettified
     * @returns Balance of the given address
     */
    _getBalance(user, prettified) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(this.tokenContract.address);
            const decimal = yield this.tokenManager.getTokenDecimals(this.tokenContract.address);
            const balance = yield this.tokenContract.balanceOf(user);
            if (prettified) {
                return new bignumber_js_1.BigNumber(balance.toString()).div(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(2);
            }
            else {
                return balance.toString();
            }
        });
    }
    mintTokens(addressToReceive, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.minterTokenContract.mint(addressToReceive, amount, Object.assign({}, options));
        });
    }
}
exports.TokenApi = TokenApi;
//# sourceMappingURL=tokenApi.js.map