import { Signer, BytesLike, ContractTransaction } from 'ethers';
import { SublimeConfig } from '../types/sublimeConfig';
import { BigNumberish } from '@ethersproject/bignumber';
import { Options as Overrides, VerifierType, Verifier as VerifierDetails, TwitterVerifierSignatureData } from '../types/Types';
/**
 * @class VerificationAPI
 */
export declare class VerificationAPI {
    /**
     * @description Signer Object
     */
    private signer;
    /**
     * @description verification contract instance
     */
    private verification;
    /**
     * @description Twitter Verifier contract instance
     */
    private twitterVerifier;
    /**
     * @description Admin Verifier contract instance
     */
    private adminVerifier;
    private personaVerifier;
    /**
     * @description Internal store for all sublime contracts addresses
     */
    private config;
    /**
     * @param signer Signer
     * @param config SublimeConfig
     */
    constructor(signer: Signer, config: SublimeConfig);
    /**
     * @param user Address of the user to be verifier
     * @param verifierType Type of Verifier
     * @returns true if the address is verified by a given verifier
     */
    isUser(user: string, verifierType: VerifierType): Promise<boolean>;
    /**
     *
     * @param verifier Address of the verifier contract
     * @returns true if the given address is a verifier
     */
    isVerifier(verifier: string): Promise<boolean>;
    /**
     * @param verifier Address of the verifier contract. Only Admin can call
     * @returns
     */
    addVerifier(verifier: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     *
     * @param _masterAddress Master Address
     * @param _isMasterLinked True if the master address is linked
     * @returns
     */
    registerMasterAddress(_masterAddress: string, _isMasterLinked: boolean, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @param _isMasterLinked True if the master address is linked
     * @param _v V component of the signature
     * @param _r R component of the signature
     * @param _s S component of the signature
     * @param _twitterId Twitter ID of the user
     * @param _deadline Deadline for the registration
     * @returns
     */
    registerSelfUsingTwitterVerifier(_isMasterLinked: boolean, _v: BigNumberish, _r: BytesLike, _s: BytesLike, _twitterId: string, _tweetId: string, _deadline: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    unregisterSelfUsingTwitterVerifier(options?: Overrides): Promise<ContractTransaction>;
    /**
     *
     * @param _isMasterLinked True if master address is linked
     * @param _v V component of the signature
     * @param _r R component of the signature
     * @param _s S component of the signature
     * @param _userData Any use metadata that is associated with the address
     * @param _deadline deadline for the transaction to be mined
     * @param options
     * @returns
     */
    registerUsingAdminVerifier(_isMasterLinked: boolean, _v: BigNumberish, _r: BytesLike, _s: BytesLike, _userData: string, _deadline: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    registerUsingPersonaVerifier(_isMasterLinked: boolean, _v: BigNumberish, _r: BytesLike, _s: BytesLike, _userData: string, _deadline: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
    unregisterSelfUsingPersonaVerifier(options?: Overrides): Promise<ContractTransaction>;
    unregisterSelfUsingAdminVerifier(options?: Overrides): Promise<ContractTransaction>;
    /**
     *
     * @param _signerAddress Address of the signer
     * @returns
     */
    updateSignerAddress(_signerAddress: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     *
     * @param hash Hash to blacklist
     * @param options
     */
    blacklistDigest(hash: BytesLike, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @param type Type of verifier
     * @returns Contract address of the given verifier
     */
    getVerifierAddress(type: VerifierType): string;
    /**
     *
     * @param address Contract address of the verifier
     * @returns Type of verifier
     */
    getVerifierType(address: string): VerifierType;
    getVerifierDisplayName(address: string): string;
    getVerifierDetails(address: string): VerifierDetails;
    getSupportedVerifiers(): VerifierDetails[];
    generateSignatureForTwitterVerifier(verifierName: string, verifierVersion: string, twitterId: string, tweetId: string, msgSender: string, timestamp: string, signerPrivate: string): Promise<TwitterVerifierSignatureData>;
    private _hashTypedDataV4_withContractAddress;
    private _domainSeparatorV4_withContractAddress;
    private _buildDomainSeparator_withContractAddress;
    private calculateDigestForTwitterVerifier;
    private keccak256;
    private addressToPaddedHex;
    private timestampPaddedHex;
    private padLeft;
}
