"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VerificationAPI = void 0;
const ethers_1 = require("ethers");
const Verification__factory_1 = require("../wrappers/factories/Verification__factory");
const TwitterVerifier__factory_1 = require("../wrappers/factories/TwitterVerifier__factory");
const AdminVerifier__factory_1 = require("../wrappers/factories/AdminVerifier__factory");
const Types_1 = require("../types/Types");
const utils_1 = require("ethers/lib/utils");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const AdminVerifierDisplayName = 'Admin Verifier';
const TwitterVerifierDisplayName = 'Twitter Verifier';
const PersonaVerifierDisplayName = 'Persona Verifier';
/**
 * @class VerificationAPI
 */
class VerificationAPI {
    /**
     * @param signer Signer
     * @param config SublimeConfig
     */
    constructor(signer, config) {
        this.signer = signer;
        this.verification = new Verification__factory_1.Verification__factory(this.signer).attach(config.verificationContractAddress);
        this.twitterVerifier = new TwitterVerifier__factory_1.TwitterVerifier__factory(this.signer).attach(config.twitterVerifierContractAddress);
        this.adminVerifier = new AdminVerifier__factory_1.AdminVerifier__factory(this.signer).attach(config.adminVerifierContractAddress);
        this.personaVerifier = new AdminVerifier__factory_1.AdminVerifier__factory(this.signer).attach(config.personaVerifierContractAddress);
        this.config = config;
    }
    /**
     * @param user Address of the user to be verifier
     * @param verifierType Type of Verifier
     * @returns true if the address is verified by a given verifier
     */
    isUser(user, verifierType) {
        return __awaiter(this, void 0, void 0, function* () {
            const verifierAddress = this.getVerifierAddress(verifierType);
            return this.verification.isUser(user, verifierAddress);
        });
    }
    /**
     *
     * @param verifier Address of the verifier contract
     * @returns true if the given address is a verifier
     */
    isVerifier(verifier) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.verification.verifiers(verifier);
        });
    }
    /**
     * @param verifier Address of the verifier contract. Only Admin can call
     * @returns
     */
    addVerifier(verifier, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.verification.addVerifier(verifier, Object.assign({}, options));
        });
    }
    /**
     *
     * @param _masterAddress Master Address
     * @param _isMasterLinked True if the master address is linked
     * @returns
     */
    registerMasterAddress(_masterAddress, _isMasterLinked, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.verification.registerMasterAddress(_masterAddress, _isMasterLinked, Object.assign({}, options));
        });
    }
    /**
     * @param _isMasterLinked True if the master address is linked
     * @param _v V component of the signature
     * @param _r R component of the signature
     * @param _s S component of the signature
     * @param _twitterId Twitter ID of the user
     * @param _deadline Deadline for the registration
     * @returns
     */
    registerSelfUsingTwitterVerifier(_isMasterLinked, _v, _r, _s, _twitterId, _tweetId, _deadline, options) {
        return this.twitterVerifier.registerSelf(_isMasterLinked, _v, _r, _s, _twitterId, _tweetId, _deadline, Object.assign({}, options));
    }
    unregisterSelfUsingTwitterVerifier(options) {
        return this.twitterVerifier.unregisterSelf(Object.assign({}, options));
    }
    /**
     *
     * @param _isMasterLinked True if master address is linked
     * @param _v V component of the signature
     * @param _r R component of the signature
     * @param _s S component of the signature
     * @param _userData Any use metadata that is associated with the address
     * @param _deadline deadline for the transaction to be mined
     * @param options
     * @returns
     */
    registerUsingAdminVerifier(_isMasterLinked, _v, _r, _s, _userData, _deadline, options) {
        return this.adminVerifier.registerSelf(_isMasterLinked, _v, _r, _s, _userData, _deadline, Object.assign({}, options));
    }
    registerUsingPersonaVerifier(_isMasterLinked, _v, _r, _s, _userData, _deadline, options) {
        return this.personaVerifier.registerSelf(_isMasterLinked, _v, _r, _s, _userData, _deadline, Object.assign({}, options));
    }
    unregisterSelfUsingPersonaVerifier(options) {
        return this.personaVerifier.unregisterSelf(Object.assign({}, options));
    }
    unregisterSelfUsingAdminVerifier(options) {
        return this.adminVerifier.unregisterSelf(Object.assign({}, options));
    }
    /**
     *
     * @param _signerAddress Address of the signer
     * @returns
     */
    updateSignerAddress(_signerAddress, options) {
        return this.twitterVerifier.updateSignerAddress(_signerAddress, Object.assign({}, options));
    }
    /**
     *
     * @param hash Hash to blacklist
     * @param options
     */
    blacklistDigest(hash, options) {
        return this.twitterVerifier.blackListDigest(hash, Object.assign({}, options));
    }
    /**
     * @param type Type of verifier
     * @returns Contract address of the given verifier
     */
    getVerifierAddress(type) {
        if (type == Types_1.VerifierType.AdminVerifier) {
            return this.config.adminVerifierContractAddress;
        }
        else if (type == Types_1.VerifierType.TwitterVerifier) {
            return this.config.twitterVerifierContractAddress;
        }
        else if (type == Types_1.VerifierType.PersonaVerifier) {
            return this.config.personaVerifierContractAddress;
        }
        else {
            return undefined;
        }
    }
    /**
     *
     * @param address Contract address of the verifier
     * @returns Type of verifier
     */
    getVerifierType(address) {
        if (!address) {
            return undefined;
        }
        if (address.toLowerCase() == this.config.adminVerifierContractAddress.toLowerCase()) {
            return Types_1.VerifierType.AdminVerifier;
        }
        else if (address.toLowerCase() == this.config.twitterVerifierContractAddress.toLowerCase()) {
            return Types_1.VerifierType.TwitterVerifier;
        }
        else if (address.toLowerCase() == this.config.personaVerifierContractAddress.toLowerCase()) {
            return Types_1.VerifierType.PersonaVerifier;
        }
        else {
            return undefined;
        }
    }
    getVerifierDisplayName(address) {
        if (!address) {
            return undefined;
        }
        if (address.toLowerCase() == this.config.adminVerifierContractAddress.toLowerCase()) {
            return AdminVerifierDisplayName;
        }
        else if (address.toLowerCase() == this.config.twitterVerifierContractAddress.toLowerCase()) {
            return TwitterVerifierDisplayName;
        }
        else if (address.toLowerCase() == this.config.personaVerifierContractAddress.toLowerCase()) {
            return PersonaVerifierDisplayName;
        }
        else {
            return undefined;
        }
    }
    getVerifierDetails(address) {
        return { address, type: this.getVerifierType(address), displayName: this.getVerifierDisplayName(address) };
    }
    getSupportedVerifiers() {
        return [Types_1.VerifierType.AdminVerifier, Types_1.VerifierType.TwitterVerifier, Types_1.VerifierType.PersonaVerifier].map((a) => {
            return { type: a, address: this.getVerifierAddress(a), displayName: this.getVerifierDisplayName(a) };
        });
    }
    generateSignatureForTwitterVerifier(verifierName, verifierVersion, twitterId, tweetId, msgSender, timestamp, signerPrivate) {
        return __awaiter(this, void 0, void 0, function* () {
            const digest = this.calculateDigestForTwitterVerifier(twitterId, tweetId, msgSender, timestamp);
            const chainId = yield (yield this.signer.getChainId()).toString();
            const hash = this._hashTypedDataV4_withContractAddress(digest, Types_1.VerifierType.TwitterVerifier, verifierName, verifierVersion, chainId);
            const signer = new ethers_1.ethers.utils.SigningKey(signerPrivate);
            const signature = signer.signDigest('0x' + hash);
            return {
                twitterId,
                tweetId,
                timestamp,
                v: signature.v,
                r: signature.r,
                s: signature.s,
                user: msgSender,
            };
        });
    }
    _hashTypedDataV4_withContractAddress(structHash, verifier, name, version, chainId) {
        const allBytesString = '1901' + this._domainSeparatorV4_withContractAddress(verifier, name, version, chainId) + structHash;
        return (0, utils_1.keccak256)('0x' + allBytesString.trim()).split('x')[1];
    }
    _domainSeparatorV4_withContractAddress(verifier, name, version, chainId) {
        const verifierAddress = this.getVerifierAddress(verifier);
        const _TYPE_HASH = this.keccak256('EIP712Domain(string name,string version,uint256 chainId,address verifyingContract)');
        const _EIP712NameHash = this.keccak256(name);
        const _EIP712VersionHash = this.keccak256(version);
        const domainSeperator = this._buildDomainSeparator_withContractAddress(_TYPE_HASH, _EIP712NameHash, _EIP712VersionHash, verifierAddress, chainId);
        return domainSeperator;
    }
    _buildDomainSeparator_withContractAddress(typeHash, nameHash, versionHash, contractAddress, chainId) {
        const allBytesString = typeHash + nameHash + versionHash + this.timestampPaddedHex(chainId) + this.addressToPaddedHex(contractAddress);
        return (0, utils_1.keccak256)('0x' + allBytesString.trim()).split('x')[1];
    }
    calculateDigestForTwitterVerifier(_twitterId, _tweetId, _user, _timestamp) {
        const allBytesString = this.keccak256('set(string twitterId,string tweetId,address userAddr,uint256 timestamp)') +
            this.keccak256(_twitterId) +
            this.keccak256(_tweetId) +
            this.addressToPaddedHex(_user) +
            this.timestampPaddedHex(_timestamp);
        return (0, utils_1.keccak256)('0x' + allBytesString.trim()).split('x')[1];
    }
    keccak256(data) {
        return (0, utils_1.keccak256)(Buffer.from(data)).split('x')[1];
    }
    addressToPaddedHex(data) {
        data = data.toLowerCase();
        const address = new bignumber_js_1.default(data).toString(16);
        return this.padLeft(address);
    }
    timestampPaddedHex(data) {
        const address = new bignumber_js_1.default(data).toString(16);
        return this.padLeft(address);
    }
    padLeft(data, size = 64) {
        const remaining = size - data.length;
        if (remaining < 0) {
            throw new Error("padded result can' be more than 32 bytes");
        }
        const pad = '0'.repeat(remaining);
        return pad + data;
    }
}
exports.VerificationAPI = VerificationAPI;
//# sourceMappingURL=verification.js.map