"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.YieldAndStrategyApi = void 0;
const bignumber_js_1 = require("bignumber.js");
const wrappers_1 = require("../wrappers");
const IYield__factory_1 = require("../wrappers/factories/IYield__factory");
const Types_1 = require("../types/Types");
const CompoundStrategyDisplayName = 'Compound Protocol';
const NoYieldStrategyDisplayName = 'Locked in Sublime';
const CompoundLogo = 'https://cryptologos.cc/logos/compound-comp-logo.png';
const SublimeLogo = 'logo pending ...';
/**
 * @class YieldAndStrategyApi
 */
class YieldAndStrategyApi {
    constructor(signer, config, tokenManger) {
        /**
         * @description Display names of all strategies
         */
        this.displayName = {};
        /**
         * @description Logo for all strategies
         */
        this.logo = {};
        this.signer = signer;
        this.config = config;
        this.tokenManager = tokenManger;
        this.compoundYield = new wrappers_1.CompoundYield__factory(this.signer).attach(this.config.compoundStrategyContractAddress);
        this.displayName[config.compoundStrategyContractAddress.toLowerCase()] = CompoundStrategyDisplayName;
        this.displayName[config.noStrategyAddress.toLowerCase()] = NoYieldStrategyDisplayName;
        this.logo[config.compoundStrategyContractAddress.toLowerCase()] = CompoundLogo;
        this.logo[config.noStrategyAddress.toLowerCase()] = SublimeLogo;
    }
    /**
     * @param yieldType Type of yield/strategy
     * @param asset Address of the asset/token
     * @param shares Number of shares
     * @param sharesInLowestUnits If true, token decimals for shares are ignored
     * @returns Total number of underlying tokens for a given asset and strategy
     */
    getTokensForShares(yieldType, asset, shares) {
        return __awaiter(this, void 0, void 0, function* () {
            const yieldAddress = this.getStrategyAddress(yieldType);
            const yieldContract = IYield__factory_1.IYield__factory.connect(yieldAddress, this.signer);
            yield this.tokenManager.updateTokenDecimals(asset);
            const _amount = new bignumber_js_1.BigNumber(shares);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('shares should be a valid number');
            }
            const _amountInLowestUnits = new bignumber_js_1.BigNumber(_amount);
            const _temp = (yield yieldContract.callStatic.getTokensForShares(_amountInLowestUnits.toFixed(0), asset)).toString();
            return new bignumber_js_1.BigNumber(_temp);
        });
    }
    /**
     * @param yieldType Type of yield/strategy
     * @param asset Address of the asset/token
     * @param amount Number of tokens
     * @returns Number of shares that will be generated for given number of tokens and strategy
     */
    getSharesForTokens(yieldType, asset, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const yieldAddress = this.getStrategyAddress(yieldType);
            const yieldContract = IYield__factory_1.IYield__factory.connect(yieldAddress, this.signer);
            const liquiditySharesAddress = yield yieldContract.liquidityToken(asset);
            yield this.tokenManager.updateTokenDecimals(liquiditySharesAddress);
            yield this.tokenManager.updateTokenDecimals(asset);
            const deopsitTokenDecimal = this.tokenManager.getTokenDecimals(asset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('shares should be a valid number');
            }
            const _temp = (yield yieldContract.callStatic.getSharesForTokens(_amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(deopsitTokenDecimal)).toFixed(0), asset)).toString();
            return new bignumber_js_1.BigNumber(_temp);
        });
    }
    getApr(yieldType, asset) {
        return __awaiter(this, void 0, void 0, function* () {
            const yieldAddress = this.getStrategyAddress(yieldType);
            const yieldContract = IYield__factory_1.IYield__factory.connect(yieldAddress, this.signer);
            if (yieldType == Types_1.StrategyType.CompoundYield) {
                const liquiditySharesAddress = yield yieldContract.liquidityToken(asset);
                const cToken = wrappers_1.ICToken__factory.connect(liquiditySharesAddress, this.signer);
                const aprString = yield (yield cToken.callStatic.supplyRatePerBlock()).toString();
                return new bignumber_js_1.BigNumber(aprString);
            }
            return new bignumber_js_1.BigNumber(0);
        });
    }
    /**
     * @description returns all strategies supported sublime-sdk
     * @returns
     */
    getStrategies() {
        return [
            {
                address: this.config.noStrategyAddress,
                type: Types_1.StrategyType.NoYield,
                displayName: this.displayName[this.config.noStrategyAddress.toLowerCase()],
                logo: this.logo[this.config.noStrategyAddress.toLowerCase()],
            },
            {
                address: this.config.compoundStrategyContractAddress,
                type: Types_1.StrategyType.CompoundYield,
                displayName: this.displayName[this.config.compoundStrategyContractAddress.toLowerCase()],
                logo: this.logo[this.config.compoundStrategyContractAddress.toLowerCase()],
            },
        ];
    }
    /**
     * @description Get Strategy address from name
     * @param strategy
     * @returns null if invalid strategy name is given
     */
    getStrategyAddress(strategy) {
        if (strategy == Types_1.StrategyType.NoYield) {
            return this.config.noStrategyAddress;
        }
        else if (strategy == Types_1.StrategyType.CompoundYield) {
            return this.config.compoundStrategyContractAddress;
        }
        else {
            return undefined;
        }
    }
    /**
     * @description Get Strategy name from given address,
     * @param address
     * @returns null if invalid strategy address is given in input
     */
    getStrategy(address) {
        if (!address) {
            return undefined;
        }
        address = address.toLowerCase();
        if (address == this.config.noStrategyAddress.toLowerCase()) {
            return Types_1.StrategyType.NoYield;
        }
        else if (address == this.config.compoundStrategyContractAddress.toLowerCase()) {
            return Types_1.StrategyType.CompoundYield;
        }
        else {
            return undefined;
        }
    }
    /**
     * @param address yield/strategy address
     * @returns URL of the strategy logo
     */
    getStrategyLogo(address) {
        if (!address) {
            return undefined;
        }
        address = address.toLowerCase();
        return this.logo[address];
    }
    /**
     * @param address yield/strategy address
     * @returns Display Name/Info for the strategy
     */
    getStrategyDisplayName(address) {
        if (!address) {
            return undefined;
        }
        address = address.toLowerCase();
        return this.displayName[address];
    }
    updateLiquidityTokenInCompoundYield(asset, cToken, options) {
        return this.compoundYield.addTokenAddress(asset, cToken, Object.assign({}, options));
    }
    setDepositLimitInCompoundYield(asset, limit, options) {
        return this.compoundYield.setDepositLimit(asset, limit.toString(), Object.assign({}, options));
    }
}
exports.YieldAndStrategyApi = YieldAndStrategyApi;
//# sourceMappingURL=yieldAndStrategy.js.map