"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreditLineEmulator = void 0;
const bignumber_js_1 = require("bignumber.js");
const Types_1 = require("../types/Types");
const Helpers_1 = require("./Helpers");
const SCALING_FACTOR = new bignumber_js_1.BigNumber(10).pow(18);
const YEAR_IN_SECONDS = new bignumber_js_1.BigNumber(86400).multipliedBy(365);
const UINT256_MAX = new bignumber_js_1.BigNumber('2').pow(256).minus(1);
class CreditLineEmulator extends Helpers_1.EmulatorHelper {
    constructor(creditLineState, externalData, globals) {
        super();
        this.creditLineState = creditLineState;
        this.externalData = externalData;
        this.globals = globals;
    }
    getId() {
        return this.creditLineState.id;
    }
    withdrawableCollateral() {
        const totalCollateralTokens = this._calculateTotalCollateralTokens();
        const _currentDebt = this.calculateCurrentDebt();
        const _collateralNeeded = _currentDebt
            .multipliedBy(this.creditLineState.idealCollateralRatio)
            .multipliedBy(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals))
            .dividedBy(this.externalData.ratioOfPrices)
            .dividedBy(SCALING_FACTOR);
        if (_collateralNeeded.gt(totalCollateralTokens)) {
            return new bignumber_js_1.BigNumber(0);
        }
        return totalCollateralTokens.minus(_collateralNeeded);
    }
    calculateTotalCollateralTokens() {
        return this._calculateTotalCollateralTokens();
    }
    _calculateTotalCollateralTokens() {
        return this.externalData.collateralPerStrategyToken.multipliedBy(this.creditLineState.collateralShareInStrategy);
    }
    calculateCurrentDebt() {
        const _interestAccrued = this.calculateInterestAccrued();
        return this.creditLineState.principal.plus(_interestAccrued).minus(this.creditLineState.totalInterestRepaid);
    }
    calculateInterestAccrued() {
        if (this.creditLineState.principal.eq(0) && this.creditLineState.lastPrincipalUpdateTime.eq(0)) {
            return new bignumber_js_1.BigNumber(0);
        }
        const _timeElapsed = this.now().minus(this.creditLineState.lastPrincipalUpdateTime);
        const _interestAccrued = this.calculateInterest(this.creditLineState.principal, this.creditLineState.borrowRate, _timeElapsed);
        return _interestAccrued.plus(this.creditLineState.interestAccruedTillLastPrincipalUpdate);
    }
    calculateInterest(principal, borrowRate, timeElapsed) {
        return principal.multipliedBy(borrowRate).multipliedBy(timeElapsed).div(SCALING_FACTOR).div(YEAR_IN_SECONDS);
    }
    calculateBorrowableAmount() {
        if (this.creditLineState.creditLineStatus != Types_1.CreditLineStatus.ACTIVE) {
            // throw new Error('CL:CBA1');
            return new bignumber_js_1.BigNumber(0);
        }
        const _collateralRatio = this.creditLineState.idealCollateralRatio;
        let _maxPossible = UINT256_MAX;
        if (_collateralRatio.gt(0)) {
            const _totalCollateralToken = this._calculateTotalCollateralTokens();
            _maxPossible = _totalCollateralToken
                .multipliedBy(this.externalData.ratioOfPrices)
                .dividedBy(_collateralRatio)
                .multipliedBy(SCALING_FACTOR)
                .div(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals));
        }
        const _currentDebt = this.calculateCurrentDebt();
        const _borrowLimit = this.creditLineState.borrowLimit;
        const _principal = this.creditLineState.principal;
        if (_maxPossible.lt(_currentDebt)) {
            return new bignumber_js_1.BigNumber(0);
        }
        return this.min(_borrowLimit.minus(_principal), _maxPossible.minus(_currentDebt));
    }
    borrowTokensToLiquidate() {
        const _currentDebt = this.calculateCurrentDebt();
        let _collateralToLiquidate = this._equivalentCollateral(_currentDebt);
        const _totalCollateralTokens = this._calculateTotalCollateralTokens();
        if (_collateralToLiquidate.gt(_totalCollateralTokens)) {
            _collateralToLiquidate = _totalCollateralTokens;
        }
        return this._borrowTokensToLiquidate(_collateralToLiquidate);
    }
    _borrowTokensToLiquidate(_collateralTokens) {
        return _collateralTokens
            .multipliedBy(SCALING_FACTOR.minus(this.globals.liquidatorRewardFraction))
            .div(SCALING_FACTOR)
            .multipliedBy(this.externalData.ratioOfPrices)
            .div(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals));
    }
    _equivalentCollateral(_borrowTokens) {
        return _borrowTokens.multipliedBy(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals)).div(this.externalData.ratioOfPrices);
    }
    calculateCurrentCollateralRatio() {
        const currentDebt = this.calculateCurrentDebt();
        const totalCollateralTokens = this.calculateTotalCollateralTokens();
        let currentCollateralRatio = UINT256_MAX;
        if (currentDebt.gt(0)) {
            currentCollateralRatio = totalCollateralTokens
                .multipliedBy(this.externalData.ratioOfPrices)
                .div(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals))
                .multipliedBy(SCALING_FACTOR)
                .div(currentDebt);
        }
        return [currentCollateralRatio, totalCollateralTokens];
    }
    getPrincipal() {
        return this.creditLineState.principal;
    }
    getCreditLimit() {
        return this.creditLineState.borrowLimit;
    }
    getStatus() {
        let _currentStatus = this.creditLineState.creditLineStatus;
        const [_currentCollateralRatio, _totalCollateralTokens] = this.calculateCurrentCollateralRatio();
        if (_currentCollateralRatio.lt(this.idealCollateralRatio())) {
            _currentStatus = Types_1.CreditLineStatus.LIQUIDATE_CALLABLE;
        }
        return _currentStatus;
    }
    // ------------------- function not part of smart contract ------------------ //
    createdAt() {
        return this.creditLineState.createdAt;
    }
    borrowAsset() {
        return this.creditLineState.borrowAsset;
    }
    collateralAsset() {
        return this.creditLineState.collateralAsset;
    }
    borrowRate() {
        return this.creditLineState.borrowRate;
    }
    idealCollateralRatio() {
        return this.creditLineState.idealCollateralRatio;
    }
    autoLiquidate() {
        return this.creditLineState.autoLiquidation;
    }
    lender() {
        return this.creditLineState.lender;
    }
    borrower() {
        return this.creditLineState.borrower;
    }
    lastPrincipalUpdateTime() {
        return this.creditLineState.lastPrincipalUpdateTime;
    }
    requestByLender() {
        return this.creditLineState.requestByLender;
    }
    borrowAssetStrategy() {
        return this.creditLineState.borrowAssetStrategy;
    }
    collateralAssetStrategy() {
        return this.creditLineState.collateralAssetStrategy;
    }
    totalInterestRepaid() {
        return this.creditLineState.totalInterestRepaid;
    }
}
exports.CreditLineEmulator = CreditLineEmulator;
//# sourceMappingURL=CreditLines.js.map