"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LenderPoolEmulator = void 0;
const bignumber_js_1 = require("bignumber.js");
const Types_1 = require("../types/Types");
const Helpers_1 = require("./Helpers");
const zero = new bignumber_js_1.BigNumber(0);
class LenderPoolEmulator extends Helpers_1.EmulatorHelper {
    constructor(lenderPoolState, lenderPoolExternalData, dataFromPooledCreditLines, lendersPerPool) {
        super();
        this.lenderData = {};
        this.borrowerInterestData = {};
        this.yieldInterestData = {};
        this.lenderPoolState = lenderPoolState;
        this.lenderPoolExternalData = lenderPoolExternalData;
        this.dataFromPooledCreditLines = dataFromPooledCreditLines;
        this.lenderPerPool = lendersPerPool;
        this.totalSupply = new bignumber_js_1.BigNumber(0);
        for (let index = 0; index < lendersPerPool.length; index++) {
            const element = lendersPerPool[index];
            this.lenderData[element.lenderAddress] = element.lenderBalance;
            this.borrowerInterestData[element.lenderAddress] = element.borrowerInterestSharesWithdrawn;
            this.yieldInterestData[element.lenderAddress] = element.yieldInterestWithdrawnShares;
            this.totalSupply = this.totalSupply.plus(element.lenderBalance);
        }
    }
    getLenderBalance(lenderAddress) {
        if (this.lenderData[lenderAddress]) {
            return this.lenderData[lenderAddress];
        }
        else {
            return new bignumber_js_1.BigNumber(0);
        }
    }
    getLenderInterest(lenderAddress) {
        const lenderBalance = this.getLenderBalance(lenderAddress);
        const borrowerInterestSharesWithdrawnByLender = this.borrowerInterestSharesWithdrawnByLender(lenderAddress);
        const yieldInterestSharesWithdrawnByLender = this.yieldInterestSharesWithdrawnByLender(lenderAddress);
        const [_borrowerInterestShares, _yieldInterestShares] = this._calculateLenderInterest(lenderBalance, this.lenderPoolState.borrowLimit, borrowerInterestSharesWithdrawnByLender, yieldInterestSharesWithdrawnByLender);
        return _borrowerInterestShares.plus(_yieldInterestShares).multipliedBy(this.lenderPoolExternalData.collateralPerStrategyToken);
    }
    calculatePrincipalWithdrawable(lenderAddress) {
        const _status = this.dataFromPooledCreditLines.status;
        const lenderBalance = this.getLenderBalance(lenderAddress); // for next use
        if (_status == Types_1.CreditLineStatus.CLOSED || _status == Types_1.CreditLineStatus.LIQUIDATED) {
            return this._calculatePrincipalWithdrawable(lenderBalance);
        }
        else if (_status == Types_1.CreditLineStatus.CANCELLED ||
            (_status == Types_1.CreditLineStatus.REQUESTED &&
                ((this.now().gte(this.lenderPoolState.startTime) && this.totalSupply.lt(this.lenderPoolState.minBorrowAmount)) ||
                    this.now().gte(this.dataFromPooledCreditLines.endsAt)))) {
            return lenderBalance;
        }
        else {
            return new bignumber_js_1.BigNumber(0);
        }
    }
    _calculateLenderInterest(_lenderBalance, _borrowLimit, borrowerInterestSharesWithdrawnByLender, yieldInterestSharesWithdrawnByLender) {
        let _totalinterestInShares = new bignumber_js_1.BigNumber(0);
        const _sharesHeld = this.lenderPoolState.sharesHeld;
        if (_sharesHeld.eq(0)) {
            return [new bignumber_js_1.BigNumber(0), new bignumber_js_1.BigNumber(0)];
        }
        const _notBorrowed = _borrowLimit.minus(this.dataFromPooledCreditLines.principal);
        const _notBorrowedInShares = _notBorrowed.dividedBy(this.lenderPoolExternalData.collateralPerStrategyToken);
        _totalinterestInShares = _sharesHeld.minus(_notBorrowedInShares);
        const _borrowerInterestShares = this.lenderPoolState.borrowerInterestShares;
        const _borrowerInterestForLender = _borrowerInterestShares
            .multipliedBy(_lenderBalance)
            .div(_borrowLimit)
            .minus(borrowerInterestSharesWithdrawnByLender);
        const _borrowerInterestSharesAfterWithdrawl = _borrowerInterestShares.minus(this.lenderPoolState.borrowerInterestSharesWithdrawn);
        const _totalYieldInterest = _totalinterestInShares
            .minus(_borrowerInterestSharesAfterWithdrawl)
            .plus(this.lenderPoolState.yieldInterestWithdrawnShares);
        const _yieldInterestForLender = _totalYieldInterest
            .multipliedBy(_lenderBalance)
            .div(_borrowLimit)
            .minus(yieldInterestSharesWithdrawnByLender);
        return [_borrowerInterestForLender, _yieldInterestForLender];
    }
    _calculatePrincipalWithdrawable(lenderBalance) {
        const _borrowTokens = this.lenderPoolState.borrowLimit;
        const _totalLiquidityWithdrawable = _borrowTokens.minus(this.dataFromPooledCreditLines.principal);
        const _principalWithdrawble = _totalLiquidityWithdrawable.multipliedBy(lenderBalance).div(_borrowTokens);
        return _principalWithdrawble;
    }
    // -------------------------------- Function not part of contract --------------------------------//
    totalInterestWithdrawnByLender(lenderAddress) {
        const borrowerInterestSharesWithdrawnByLender = this.borrowerInterestSharesWithdrawnByLender(lenderAddress);
        const yieldInterestSharesWithdrawnByLender = this.yieldInterestSharesWithdrawnByLender(lenderAddress);
        return borrowerInterestSharesWithdrawnByLender
            .plus(yieldInterestSharesWithdrawnByLender)
            .multipliedBy(this.lenderPoolExternalData.collateralPerStrategyToken);
    }
    borrowerInterestSharesWithdrawnByLender(lenderAddress) {
        if (this.borrowerInterestData[lenderAddress]) {
            return this.borrowerInterestData[lenderAddress];
        }
        else {
            return new bignumber_js_1.BigNumber(0);
        }
    }
    yieldInterestSharesWithdrawnByLender(lenderAddress) {
        if (this.yieldInterestData[lenderAddress]) {
            return this.yieldInterestData[lenderAddress];
        }
        else {
            return new bignumber_js_1.BigNumber(0);
        }
    }
    getAllLenders() {
        return this.lenderPerPool;
    }
    getPrincipalWithdrawableForAllLenders() {
        return this.lenderPerPool.map((a) => {
            return { lenderAddress: a.lenderAddress, principalWithdrawable: this.calculatePrincipalWithdrawable(a.lenderAddress) };
        });
    }
    // ------ calls not part of contract  ------------ //
    getLenderData(lenderAddress) {
        const lenderData = this.getAllLenders().filter((a) => a.lenderAddress.toLowerCase() === lenderAddress.toLowerCase());
        if (lenderData.length == 0) {
            return {
                lenderAddress: lenderAddress,
                amountLent: zero,
                lenderBalance: zero,
                borrowerInterestSharesWithdrawn: zero,
                yieldInterestWithdrawnShares: zero,
                amountWithdrawn: zero,
                sharesWithdrawn: zero,
                interestWithdrawn: zero,
            };
        }
        else {
            return lenderData[0];
        }
    }
    totalPrincipalSuppliedByLender(lenderAddress) {
        return this.getLenderData(lenderAddress).amountLent;
    }
    withdrawableLiquidity(lenderAddress) {
        return this.calculateWithdrawableLiquidity(lenderAddress, false);
    }
    withdrawableLiquidatedCollateral(lenderAddress) {
        return this.calculateWithdrawableLiquidity(lenderAddress, true);
    }
    calculateWithdrawableLiquidity(lenderAddress, _isLiquidationWithdrawn) {
        const _liquidityProvided = this.getLenderBalance(lenderAddress);
        if (_liquidityProvided.eq(0)) {
            return zero;
        }
        let _status = this.dataFromPooledCreditLines.status;
        // const _borrowAsset = pooledCLConstants[_id].borrowAsset;
        if (_status == Types_1.CreditLineStatus.REQUESTED) {
            if (this.now().gt(this.dataFromPooledCreditLines.startTime) && this.totalSupply.lt(this.lenderPoolState.minBorrowAmount)) {
            }
            else if (this.now().gte(this.dataFromPooledCreditLines.endsAt)) {
            }
            else {
                return zero;
            }
            _status = Types_1.CreditLineStatus.CANCELLED;
            //     delete pooledCLConstants[_id].startTime;
        }
        if (_status == Types_1.CreditLineStatus.CANCELLED) {
            return _liquidityProvided;
        }
        else if (_status == Types_1.CreditLineStatus.CLOSED || _status == Types_1.CreditLineStatus.LIQUIDATED) {
            if (_status == Types_1.CreditLineStatus.LIQUIDATED) {
                if (_isLiquidationWithdrawn) {
                    return zero;
                }
            }
            // all other cases distribute the sharesHeld proportional to their poolToken balances
            // address _strategy = pooledCLConstants[_id].borrowAssetStrategy;
            const _principalWithdrawable = this.calculatePrincipalWithdrawable(lenderAddress);
            const _interestSharesWithdrawable = this._updateInterestSharesToWithdraw(lenderAddress);
            const _interestWithdrawable = _interestSharesWithdrawable.multipliedBy(this.lenderPoolExternalData.collateralPerStrategyToken);
            const _amountToWithdraw = _principalWithdrawable.plus(_interestWithdrawable);
            const _sharesToWithdraw = _amountToWithdraw.dividedBy(this.lenderPoolExternalData.collateralPerStrategyToken);
            return _sharesToWithdraw;
        }
        else {
            return zero;
        }
    }
    _updateInterestSharesToWithdraw(lenderAddress) {
        const _lenderBalance = this.getLenderBalance(lenderAddress);
        if (_lenderBalance.eq(0)) {
            return zero;
        }
        const borrowerInterestSharesWithdrawnByLender = this.borrowerInterestSharesWithdrawnByLender(lenderAddress);
        const yieldInterestSharesWithdrawnByLender = this.yieldInterestSharesWithdrawnByLender(lenderAddress);
        const [_borrowerInterestForLender, _yieldInterestForLender] = this._calculateLenderInterest(_lenderBalance, this.lenderPoolState.borrowLimit, borrowerInterestSharesWithdrawnByLender, yieldInterestSharesWithdrawnByLender);
        return _yieldInterestForLender.plus(_borrowerInterestForLender);
    }
}
exports.LenderPoolEmulator = LenderPoolEmulator;
//# sourceMappingURL=LenderPool.js.map