"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PooledCreditLineEmulator = void 0;
const bignumber_js_1 = require("bignumber.js");
const Types_1 = require("../types/Types");
const Helpers_1 = require("./Helpers");
const LenderPool_1 = require("./LenderPool");
const SCALING_FACTOR = new bignumber_js_1.BigNumber(10).pow(18);
const YEAR_IN_SECONDS = new bignumber_js_1.BigNumber(86400).multipliedBy(365);
const UINT256_MAX = new bignumber_js_1.BigNumber('2').pow(256).minus(1);
class PooledCreditLineEmulator extends Helpers_1.EmulatorHelper {
    constructor(pooledCreditLineState, externalData, lenderPoolState, lendersPerPool) {
        super();
        this.pooledCreditLineState = pooledCreditLineState;
        this.externalData = externalData;
        this.lenderPoolState = lenderPoolState;
        this.lenderPoolExternalData = externalData;
        this.lendersPerPool = lendersPerPool;
    }
    getId() {
        return this.pooledCreditLineState.id;
    }
    getLenderPoolEmulator() {
        return new LenderPool_1.LenderPoolEmulator(this.lenderPoolState, this.lenderPoolExternalData, {
            principal: this.pooledCreditLineState.principal,
            status: this.getStatusAndUpdate(),
            endsAt: this.pooledCreditLineState.endsAt,
            startTime: this.pooledCreditLineState.startsAt,
        }, this.lendersPerPool);
    }
    withdrawableCollateral() {
        const _status = this.getStatusAndUpdate();
        if ([Types_1.CreditLineStatus.EXPIRED, Types_1.CreditLineStatus.CANCELLED, Types_1.CreditLineStatus.REQUESTED].includes(_status)) {
            return new bignumber_js_1.BigNumber(0);
        }
        const _totalCollateralTokens = this.calculateTotalCollateralTokens();
        if ([Types_1.CreditLineStatus.LIQUIDATED, Types_1.CreditLineStatus.CLOSED].includes(_status)) {
            return _totalCollateralTokens;
        }
        const _currentDebt = this.calculateCurrentDebt();
        const _collateralRatio = this.pooledCreditLineState.idealCollateralRatio;
        const _collateralNeeded = _currentDebt
            .multipliedBy(_collateralRatio)
            .div(this.externalData.ratioOfPrices)
            .multipliedBy(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals))
            .div(SCALING_FACTOR);
        if (_collateralNeeded.gte(_totalCollateralTokens)) {
            return new bignumber_js_1.BigNumber(0);
        }
        return _totalCollateralTokens.minus(_collateralNeeded);
    }
    calculateInterestAccrued() {
        const _lastPrincipalUpdateTime = this.pooledCreditLineState.lastPrincipalUpdateTime;
        const _principal = this.pooledCreditLineState.principal;
        if (_lastPrincipalUpdateTime.eq(0) && _principal.eq(0)) {
            return new bignumber_js_1.BigNumber(0);
        }
        const _timeElapsed = this.now().minus(_lastPrincipalUpdateTime);
        const _endTime = this.pooledCreditLineState.endsAt;
        const _penaltyRate = this.pooledCreditLineState.gracePenaltyRate;
        const _borrowRate = this.pooledCreditLineState.borrowRate;
        let _penaltyInterest = new bignumber_js_1.BigNumber(0);
        if (_lastPrincipalUpdateTime.lte(_endTime) && this.now().gt(_endTime)) {
            _penaltyInterest = this.calculateInterest(_principal, _penaltyRate, this.now().minus(_endTime));
        }
        else {
            _penaltyInterest = this.calculateInterest(_principal, _penaltyRate, this.now().minus(_lastPrincipalUpdateTime));
        }
        let _interestAccrued = this.calculateInterest(_principal, _borrowRate, _timeElapsed);
        _interestAccrued = _interestAccrued.plus(_penaltyInterest);
        return _interestAccrued.plus(this.pooledCreditLineState.interestAccruedTillLastPrincipalUpdate);
    }
    calculateCurrentDebt() {
        const _interestAccrued = this.calculateInterestAccrued();
        // console.log("_interestAccrued", _interestAccrued.toFixed());
        // console.log("this.pooledCreditLineState.principal", this.pooledCreditLineState.principal.toFixed());
        // console.log("this.pooledCreditLineState.totalInterestRepaid", this.pooledCreditLineState.totalInterestRepaid.toFixed());
        return this.pooledCreditLineState.principal.plus(_interestAccrued).minus(this.pooledCreditLineState.totalInterestRepaid);
    }
    calculateTotalCollateralTokens() {
        return this.externalData.collateralPerStrategyToken.multipliedBy(this.pooledCreditLineState.depositedCollateralInShares);
    }
    calculateInterest(principal, borrowRate, timeElapsed) {
        return principal.multipliedBy(borrowRate).multipliedBy(timeElapsed).div(SCALING_FACTOR).div(YEAR_IN_SECONDS);
    }
    getRequiredCollateral(_borrowTokennAmount) {
        const _collateral = this._equivalentCollateral(_borrowTokennAmount);
        return _collateral.multipliedBy(this.pooledCreditLineState.idealCollateralRatio).div(SCALING_FACTOR);
    }
    _equivalentCollateral(_borrowTokennAmount) {
        return _borrowTokennAmount
            .multipliedBy(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals))
            .div(this.externalData.ratioOfPrices);
    }
    collateralTokensToLiquidate(_borrowTokensToLiquidate) {
        return this._equivalentCollateral(_borrowTokensToLiquidate);
    }
    calculateBorrowableAmount() {
        const _status = this.getStatusAndUpdate();
        if (_status != Types_1.CreditLineStatus.ACTIVE)
            return new bignumber_js_1.BigNumber(0);
        const _totalCollateralTokens = this.calculateTotalCollateralTokens();
        const _currentDebt = this.calculateCurrentDebt();
        const _collateralRatio = this.pooledCreditLineState.idealCollateralRatio;
        let _maxPossible = UINT256_MAX;
        if (_collateralRatio.gt(0)) {
            _maxPossible = _totalCollateralTokens
                .multipliedBy(this.externalData.ratioOfPrices)
                .div(_collateralRatio)
                .multipliedBy(SCALING_FACTOR)
                .div(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals));
        }
        const _borrowLimit = this.pooledCreditLineState.borrowLimit;
        const _principal = this.pooledCreditLineState.principal;
        if (_maxPossible.lte(_currentDebt))
            return new bignumber_js_1.BigNumber(0);
        return this.min(_borrowLimit.minus(_principal), _maxPossible.minus(_currentDebt));
    }
    calculateCurrentCollateralRatio() {
        const _currentDebt = this.calculateCurrentDebt();
        let _currentCollateralRatio = UINT256_MAX;
        if (_currentDebt.gt(0)) {
            _currentCollateralRatio = this.calculateTotalCollateralTokens()
                .multipliedBy(this.externalData.ratioOfPrices)
                .div(_currentDebt)
                .multipliedBy(SCALING_FACTOR)
                .div(new bignumber_js_1.BigNumber(10).pow(this.externalData.ratioOfPricesDecimals));
        }
        return _currentCollateralRatio;
    }
    getStatusAndUpdate() {
        let currentStatus = this.pooledCreditLineState.pooledCreditLineStatus;
        if (currentStatus == Types_1.CreditLineStatus.ACTIVE && this.pooledCreditLineState.endsAt.lte(this.now())) {
            if (this.pooledCreditLineState.principal.gt(0)) {
                currentStatus = Types_1.CreditLineStatus.EXPIRED;
            }
            else {
                currentStatus = Types_1.CreditLineStatus.CLOSED;
            }
            this.pooledCreditLineState.pooledCreditLineStatus = currentStatus;
        }
        return currentStatus;
    }
    // ----------- additional calls not part of smart contract ----------//
    getStatus() {
        const currentStatus = this.pooledCreditLineState.pooledCreditLineStatus;
        const _idealCollateralRatio = this.pooledCreditLineState.idealCollateralRatio;
        if (currentStatus == Types_1.CreditLineStatus.ACTIVE) {
            if (this.pooledCreditLineState.endsAt.lte(this.now())) {
                // normal loan duration has ended
                if (this.pooledCreditLineState.principal.gt(0)) {
                    // PCL principal is non-zero, so goes to expired state
                    if (this.pooledCreditLineState.defaultsAt.lte(this.now())) {
                        // PCL has crossed grace period, can liquidate
                        return Types_1.CreditLineStatus.DEFAULT_LIQUIDATE_CALLABLE;
                    }
                    else
                        return Types_1.CreditLineStatus.EXPIRED; // PCL is in grace period and collat. ratio is OK
                }
                else {
                    // PCL principal is zero, so can be closed
                    return Types_1.CreditLineStatus.CLOSE_CALLABLE;
                }
            }
            if (_idealCollateralRatio.gt(0)) {
                // PCL has non-zero collat. requirement
                const currentCollateralRatio = this.calculateCurrentCollateralRatio();
                if (currentCollateralRatio.lt(_idealCollateralRatio)) {
                    // active PCL and collat ratio is below threshold
                    return Types_1.CreditLineStatus.LIQUIDATE_CALLABLE;
                }
            }
        }
        else if (currentStatus == Types_1.CreditLineStatus.EXPIRED) {
            if (this.pooledCreditLineState.principal.eq(0)) {
                // loan has been repaid, can be closed
                return Types_1.CreditLineStatus.CLOSE_CALLABLE;
            }
            else if (this.pooledCreditLineState.defaultsAt.lte(this.now())) {
                // principal not zero, & grace period is over
                return Types_1.CreditLineStatus.DEFAULT_LIQUIDATE_CALLABLE;
            }
            else if (_idealCollateralRatio.gt(0)) {
                const currentCollateralRatio = this.calculateCurrentCollateralRatio();
                if (currentCollateralRatio.lt(_idealCollateralRatio)) {
                    // collat. ratio is not OK
                    return Types_1.CreditLineStatus.LIQUIDATE_CALLABLE;
                }
            }
        }
        else if (currentStatus == Types_1.CreditLineStatus.REQUESTED) {
            if (this.pooledCreditLineState.startsAt.lte(this.now())) {
                if (this.pooledCreditLineState.endsAt.lte(this.now())) {
                    // loan not started and beyond loan end time
                    return Types_1.CreditLineStatus.INTERMEDIATE_CANCELLED;
                }
                else if (this.pooledCreditLineState.minBorrowAmount.lte(this.totalAmountLent())) {
                    // before loan end time & min amount has been met
                    return Types_1.CreditLineStatus.START_CALLABLE;
                }
                else if (this.pooledCreditLineState.minBorrowAmount.gt(this.totalAmountLent())) {
                    // before loan end time & min amount has not been met
                    return Types_1.CreditLineStatus.INTERMEDIATE_CANCELLED;
                }
            }
        }
        return currentStatus;
    }
    getPrincipal() {
        return this.pooledCreditLineState.principal;
    }
    getBorrowLimit() {
        return this.pooledCreditLineState.borrowLimit;
    }
    totalAmountLent() {
        return this.lendersPerPool.reduce((total, current) => total.plus(current.amountLent), new bignumber_js_1.BigNumber(0));
    }
    totalSupply() {
        return this.lendersPerPool.reduce((total, current) => total.plus(current.lenderBalance), new bignumber_js_1.BigNumber(0));
    }
    borrowerAddress() {
        return this.pooledCreditLineState.borrowerAddress;
    }
    borrowRate() {
        return this.pooledCreditLineState.borrowRate;
    }
    idealCollateralratio() {
        return this.pooledCreditLineState.idealCollateralRatio;
    }
    borrowAsset() {
        return this.pooledCreditLineState.borrowAsset;
    }
    collateralAsset() {
        return this.pooledCreditLineState.collateralAsset;
    }
    createdAt() {
        return this.pooledCreditLineState.createdAt;
    }
    startsAt() {
        return this.pooledCreditLineState.startsAt;
    }
    endsAt() {
        return this.pooledCreditLineState.endsAt;
    }
    defaultsAt() {
        return this.pooledCreditLineState.defaultsAt;
    }
    lenderStrategy() {
        return this.pooledCreditLineState.lenderStrategy;
    }
    collateralStrategy() {
        return this.pooledCreditLineState.collateralStrategy;
    }
    gracePenaltyRate() {
        return this.pooledCreditLineState.gracePenaltyRate;
    }
    totalInterestRepaid() {
        return this.pooledCreditLineState.totalInterestRepaid;
    }
    lastPrincipalUpdateTime() {
        return this.pooledCreditLineState.lastPrincipalUpdateTime;
    }
    interestAccruedTillLastPrincipalUpdate() {
        return this.pooledCreditLineState.interestAccruedTillLastPrincipalUpdate;
    }
    totalLentAmount() {
        return this.pooledCreditLineState.totalLentAmount;
    }
    minBorrowAmount() {
        return this.pooledCreditLineState.minBorrowAmount;
    }
    depositedCollateralInShares() {
        return this.pooledCreditLineState.depositedCollateralInShares;
    }
    borrowerVerifier() {
        return this.pooledCreditLineState.borrowerVerifier;
    }
    lenderVerifier() {
        return this.pooledCreditLineState.lenderVerifier;
    }
    ratioOfCollateralTokensVsBorrowTokensPrice() {
        return {
            value: this.externalData.ratioOfPrices.toString(),
            decimals: this.externalData.ratioOfPricesDecimals,
        };
    }
}
exports.PooledCreditLineEmulator = PooledCreditLineEmulator;
//# sourceMappingURL=PooledCreditLines.js.map