import { Base } from './Base';
import { Signer } from '@ethersproject/abstract-signer';
import { TokenManager } from '../tokenManager';
import { SublimeConfig } from '../types/sublimeConfig';
import { CreditLineDetail, CreditLinesOverview, CreditLineOperation, CreditLineStatus } from '../types/Types';
export declare class CreditLineCalls extends Base {
    /**
     * @description instance of credit line contract
     */
    constructor(url: string, signer: Signer, tokenManager: TokenManager, config: SublimeConfig);
    /**
     * @description use to fetch all the credit lines in the system
     * @description don't use in production. It will be a 😰 (only for debugging)
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @returns
     */
    getAllCreditLines(count?: number, skip?: number): Promise<CreditLineDetail[]>;
    private convertToCreditLineStatusEnum;
    /**
     *
     * @param data
     * @description transaform the data recevied from the subgraph to type
     */
    private transformToCreditLine;
    /**
     * @param borrower: address of the borrower
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the confirmed credit lines of a borrower
     */
    getConfirmedCreditLinesOfBorrower(borrower: string, count?: number, skip?: number): Promise<CreditLineDetail[]>;
    /**
     * @param lender: address of the lender
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the confirmed credit lines of a lender
     */
    getConfirmedCreditLinesOfLender(lender: string, count?: number, skip?: number): Promise<CreditLineDetail[]>;
    /**
     * @param lender: address of the lender
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of credit lines which have been requested by the lender but not accepted by any borrower
     */
    getPendingCreditlinesRequestedByLender(lender: string, count: number, skip: number): Promise<CreditLineDetail[]>;
    /**
     * @param borrower: address of the borrower
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of credit lines which have been requests by the borrower but not accpeted by any lender
     */
    getPendingCreditLinesRequestedByBorrower(borrower: string, count: number, skip: number): Promise<CreditLineDetail[]>;
    /**
     * @param lender: address of the lender
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of poending credit lines which have requested to a lender by all other borrowers
     */
    getPendingCreditLinesRequestedToLender(lender: string, count: number, skip: number): Promise<CreditLineDetail[]>;
    /**
     * @param borrower: address of the borrower
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of poending credit lines which have requested to a borrower by all other lenders
     */
    getPendingCreditLinesRequestedToBorrower(borrower: string, count: number, skip: number): Promise<CreditLineDetail[]>;
    /**
     * @param id
     * @description returns a detailed information of a single credit line.
     */
    getCreditLine(id: string): Promise<CreditLineDetail>;
    /**
     * @param address: address of the user to query
     * @description Returns the credit lines overview of the user
     */
    getCreditLinesOverview(address: string, count?: number, skip?: number): Promise<CreditLinesOverview>;
    /**
     * @param creditLine
     * @description returns the operations made on a credit line in time-wise order
     */
    getCreditLineTimeline(creditLine: string): Promise<CreditLineOperation[]>;
    /**
     *
     * @param cl
     * @description Transforms the data received from the subgraph into type
     */
    private transformToCreditLineOperations;
    countAllCreditLines(): Promise<number>;
    countAllCreditLinesOfBorrowerWithStateIn(borrower: string, state: CreditLineStatus[]): Promise<number>;
    countAllCreditLinesOfBorrowerWithStateIn_requestByLender(borrower: string, requestByLender: boolean, state: CreditLineStatus[]): Promise<number>;
    countAllCreditLinesOfBorrowerWithStateNotIn(borrower: string, state: CreditLineStatus[]): Promise<number>;
    countAllCreditLinesOfBorrowerWithStateNotIn_requestByLender(borrower: string, requestByLender: boolean, state: CreditLineStatus[]): Promise<number>;
    countAllCreditLinesOfLenderWithStateNotIn(lender: string, state: CreditLineStatus[]): Promise<number>;
    countAllCreditLinesOfLenderWithStateNotIn_requestByLender(lender: string, requestByLender: boolean, state: CreditLineStatus[]): Promise<number>;
    countAllCreditLinesOfLenderWithStateIn(lender: string, state: CreditLineStatus[]): Promise<number>;
    countAllCreditLinesOfLenderWithStateIn_requestByLender(lender: string, requestByLender: boolean, state: CreditLineStatus[]): Promise<number>;
    private transformToCreditLineEmulator;
}
