"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreditLineCalls = void 0;
const Base_1 = require("./Base");
const queries_1 = require("../queries");
const Types_1 = require("../types/Types");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const CreditLines_1 = require("../emulator/CreditLines");
class CreditLineCalls extends Base_1.Base {
    /**
     * @description instance of credit line contract
     */
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    /**
     * @description use to fetch all the credit lines in the system
     * @description don't use in production. It will be a 😰 (only for debugging)
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @returns
     */
    getAllCreditLines(count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllCreditLinesFromSubgraph)(this.subgraphUrl, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            return this.transformToCreditLine(emulatorResult, prices);
        });
    }
    convertToCreditLineStatusEnum(stateInSubgraph) {
        // enum CreditLineStatus {
        //     NOT_CREATED,
        //     REQUESTED,
        //     ACTIVE
        // }
        if (stateInSubgraph == 'NOT_CREATED') {
            return Types_1.CreditLineStatus.NOT_CREATED;
        }
        else if (stateInSubgraph == 'REQUESTED') {
            return Types_1.CreditLineStatus.REQUESTED;
        }
        else if (stateInSubgraph == 'ACTIVE') {
            return Types_1.CreditLineStatus.ACTIVE;
        }
        else {
            return Types_1.CreditLineStatus.CLOSED;
        }
    }
    /**
     *
     * @param data
     * @description transaform the data recevied from the subgraph to type
     */
    transformToCreditLine(emulatorResult, prices) {
        return emulatorResult.map((aNew) => {
            return {
                createdAt: aNew.createdAt().toString(),
                currentDebt: { value: aNew.calculateCurrentDebt().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                principal: { value: aNew.getPrincipal().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                interestAccrued: {
                    value: aNew.calculateInterestAccrued().toString(),
                    decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()),
                },
                collateralRatio: { value: aNew.calculateCurrentCollateralRatio()[0].toString(), decimals: 18 },
                creditLimit: { value: aNew.getCreditLimit().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                interestRate: { value: aNew.borrowRate().toString(), decimals: 18 },
                idealCollateralRatio: { value: aNew.idealCollateralRatio().toString(), decimals: 18 },
                borrowAsset: {
                    address: aNew.borrowAsset(),
                    name: this.tokenManager.getTokenName(aNew.borrowAsset()),
                    pricePerAssetInUSD: prices[aNew.borrowAsset()].toString(),
                    logo: this.tokenManager.getLogo(aNew.borrowAsset()),
                    decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()),
                },
                collateralTokens: {
                    value: aNew.calculateTotalCollateralTokens().toString(),
                    decimals: this.tokenManager.getTokenDecimals(aNew.collateralAsset()),
                },
                collateralAsset: {
                    address: aNew.collateralAsset(),
                    name: this.tokenManager.getTokenName(aNew.collateralAsset()),
                    pricePerAssetInUSD: prices[aNew.collateralAsset()].toString(),
                    logo: this.tokenManager.getLogo(aNew.collateralAsset()),
                    decimals: this.tokenManager.getTokenDecimals(aNew.collateralAsset()),
                },
                autoLiquidate: aNew.autoLiquidate(),
                lender: { address: aNew.lender() },
                borrower: { address: aNew.borrower() },
                type: aNew.getStatus(),
                lastPrincipalUpdateTime: aNew.lastPrincipalUpdateTime().toString(),
                id: aNew.getId(),
                requestByLender: aNew.requestByLender(),
                borrowAssetStrategy: {
                    address: aNew.borrowAssetStrategy(),
                    type: this.yieldApi.getStrategy(aNew.borrowAssetStrategy()),
                    displayName: this.yieldApi.getStrategyDisplayName(aNew.borrowAssetStrategy()),
                    logo: this.yieldApi.getStrategyLogo(aNew.borrowAssetStrategy()),
                },
                collateralAssetStrategy: {
                    address: aNew.collateralAssetStrategy(),
                    type: this.yieldApi.getStrategy(aNew.collateralAssetStrategy()),
                    displayName: this.yieldApi.getStrategyDisplayName(aNew.collateralAssetStrategy()),
                    logo: this.yieldApi.getStrategyLogo(aNew.collateralAssetStrategy()),
                },
                emulator: aNew,
            };
        });
    }
    /**
     * @param borrower: address of the borrower
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the confirmed credit lines of a borrower
     */
    getConfirmedCreditLinesOfBorrower(borrower, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getConfirmedCreditLinesOfBorrower)(this.subgraphUrl, borrower, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            return this.transformToCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @param lender: address of the lender
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the confirmed credit lines of a lender
     */
    getConfirmedCreditLinesOfLender(lender, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getConfirmedCreditLinesOfLender)(this.subgraphUrl, lender, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            return this.transformToCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @param lender: address of the lender
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of credit lines which have been requested by the lender but not accepted by any borrower
     */
    getPendingCreditlinesRequestedByLender(lender, count, skip) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPendingCreditlinesRequestedByLender)(this.subgraphUrl, lender, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            return this.transformToCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @param borrower: address of the borrower
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of credit lines which have been requests by the borrower but not accpeted by any lender
     */
    getPendingCreditLinesRequestedByBorrower(borrower, count, skip) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPendingCreditLinesRequestedByBorrower)(this.subgraphUrl, borrower, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            return this.transformToCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @param lender: address of the lender
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of poending credit lines which have requested to a lender by all other borrowers
     */
    getPendingCreditLinesRequestedToLender(lender, count, skip) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPendingCreditLinesRequestedToLender)(this.subgraphUrl, lender, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            return this.transformToCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @param borrower: address of the borrower
     * @param count: number of credit lines to query
     * @param skip: number to credit lines to skip
     * @description Returns the list of poending credit lines which have requested to a borrower by all other lenders
     */
    getPendingCreditLinesRequestedToBorrower(borrower, count, skip) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPendingCreditLinesRequestedToBorrower)(this.subgraphUrl, borrower, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            return this.transformToCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @param id
     * @description returns a detailed information of a single credit line.
     */
    getCreditLine(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLine)(this.subgraphUrl, id);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            const emulatorResult = this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
            const data = this.transformToCreditLine(emulatorResult, prices);
            if (data.length == 0) {
                return undefined;
            }
            else {
                return data[0];
            }
        });
    }
    /**
     * @param address: address of the user to query
     * @description Returns the credit lines overview of the user
     */
    getCreditLinesOverview(address, count = 999, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const creditLineAsborrower = yield this.getConfirmedCreditLinesOfBorrower(address, count, skip);
            const creditLineAsLender = yield this.getConfirmedCreditLinesOfLender(address, count, skip);
            // const creditLineAsborrower: CreditLineDetail[] = [];
            // const creditLineAsLender: CreditLineDetail[] = [];
            let creditGranted = new bignumber_js_1.default(0);
            let interestAccrued = new bignumber_js_1.default(0);
            let activeCredit = new bignumber_js_1.default(0);
            let interestRate = new bignumber_js_1.default(0);
            for (let index = 0; index < creditLineAsLender.length; index++) {
                const element = creditLineAsLender[index];
                const emulator = element.emulator;
                if ([Types_1.CreditLineStatus.CLOSED, Types_1.CreditLineStatus.LIQUIDATED].includes(emulator.getStatus())) {
                    continue;
                }
                const borrowTokenDecimals = yield this.tokenManager.getTokenDecimals(emulator.borrowAsset());
                const assetPrice = yield this.tokenManager.getPricePerAsset(emulator.borrowAsset());
                const principal = emulator.getPrincipal().div(new bignumber_js_1.default(10).pow(borrowTokenDecimals)).multipliedBy(assetPrice);
                creditGranted = creditGranted.plus(principal);
            }
            // for (let i = 0; i < creditLineAsborrower.length; i++) {
            //   const interest = creditLineAsborrower[i].interestRate;
            //   const accruedInterest = creditLineAsborrower[i].interestAccrued.value;
            //   if (creditLineAsborrower[i].type == 'ACTIVE') {
            //     const credit = new BigNumber(creditLineAsborrower[i].principal.value.toString());
            //     const price = await this.tokenManager.getPricePerAsset(creditLineAsLender[i].borrowAsset.toString());
            //     activeCredit = activeCredit.plus(credit.multipliedBy(price));
            //   }
            //   interestAccrued = interestAccrued.plus(accruedInterest.toString());
            //   interestRate = interestRate.plus(new BigNumber(interest.value.toString()).dividedBy('1000000000000000000'));
            // }
            let totalPrincipal = new bignumber_js_1.default(0);
            for (let index = 0; index < creditLineAsborrower.length; index++) {
                const element = creditLineAsborrower[index];
                const emulator = element.emulator;
                const accruedInterest = emulator.calculateInterestAccrued();
                const borrowTokenDecimals = yield this.tokenManager.getTokenDecimals(emulator.borrowAsset());
                const price = yield this.tokenManager.getPricePerAsset(emulator.borrowAsset());
                if ([Types_1.CreditLineStatus.ACTIVE].includes(emulator.getStatus())) {
                    const credit = emulator.getPrincipal().div(new bignumber_js_1.default(10).pow(borrowTokenDecimals)).multipliedBy(price);
                    activeCredit = activeCredit.plus(credit);
                }
                interestAccrued = interestAccrued.plus(accruedInterest.div(new bignumber_js_1.default(10).pow(borrowTokenDecimals)).multipliedBy(price));
                totalPrincipal = totalPrincipal.plus(emulator.getPrincipal().div(new bignumber_js_1.default(10).pow(borrowTokenDecimals)).multipliedBy(price));
            }
            if (totalPrincipal.gt(0)) {
                interestRate = interestAccrued.div(totalPrincipal).multipliedBy(100);
            }
            return {
                creditGranted: { value: creditGranted.toFixed(2), decimals: 2 },
                interestAccrued: { value: interestAccrued.toFixed(2), decimals: 2 },
                activeCredit: { value: activeCredit.toFixed(2), decimals: 2 },
                interestRate: interestRate.toFixed(2),
            };
        });
    }
    /**
     * @param creditLine
     * @description returns the operations made on a credit line in time-wise order
     */
    getCreditLineTimeline(creditLine) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLineTimeline)(this.subgraphUrl, creditLine);
            if (result.data.creditLines.length == 0) {
                return [];
            }
            else {
                const cl = result.data.creditLines[0];
                return yield this.transformToCreditLineOperations(cl);
            }
        });
    }
    /**
     *
     * @param cl
     * @description Transforms the data received from the subgraph into type
     */
    transformToCreditLineOperations(cl) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateAll(cl.borrowAsset);
            yield this.tokenManager.updateAll(cl.collateralAsset);
            const operations = cl.creditLineTimeline.map((a) => {
                // NOT_CREATED
                // REQUESTED
                // ACTIVE
                // DEPOSIT_COLLATERAL
                // BORROW
                // REPAY
                // WITHDRAW_COLLATERAL
                // CLOSED
                // CANCELLED
                // RESET
                // LIQUIDATED
                let decimalsToUse = 0;
                if (['DEPOSIT_COLLATERAL', 'WITHDRAW_COLLATERAL'].includes(a.creditLineOperation)) {
                    decimalsToUse = this.tokenManager.getTokenDecimals(cl.collateralAsset);
                }
                if (['BORROW', 'REPAY'].includes(a.creditLineOperation)) {
                    decimalsToUse = this.tokenManager.getTokenDecimals(cl.borrowAsset);
                }
                return {
                    amount: {
                        value: a.amount || '0',
                        decimals: decimalsToUse,
                    },
                    creditLineOperation: a.creditLineOperation,
                    liquidator: a.liquidator,
                    strategy: a.strategy ? { type: this.yieldApi.getStrategy(a.strategy), address: a.strategy } : undefined,
                    timestamp: a.timestamp,
                    id: a.id,
                };
            });
            return operations;
        });
    }
    countAllCreditLines() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllCreditLinesForCount)(this.subgraphUrl);
            return result.length;
        });
    }
    countAllCreditLinesOfBorrowerWithStateIn(borrower, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfBorrowerWithState)(this.subgraphUrl, borrower, state.map((a) => a.toString()));
            return result.length;
        });
    }
    countAllCreditLinesOfBorrowerWithStateIn_requestByLender(borrower, requestByLender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfBorrowerWithState_requestByLender)(this.subgraphUrl, borrower, requestByLender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    countAllCreditLinesOfBorrowerWithStateNotIn(borrower, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfBorrowerWithStateNotIn)(this.subgraphUrl, borrower, state.map((a) => a.toString()));
            return result.length;
        });
    }
    countAllCreditLinesOfBorrowerWithStateNotIn_requestByLender(borrower, requestByLender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfBorrowerWithStateNotIn_requestByLender)(this.subgraphUrl, borrower, requestByLender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    countAllCreditLinesOfLenderWithStateNotIn(lender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfLenderWithStateNotIn)(this.subgraphUrl, lender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    countAllCreditLinesOfLenderWithStateNotIn_requestByLender(lender, requestByLender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfLenderWithStateNotIn_requestByLender)(this.subgraphUrl, lender, requestByLender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    countAllCreditLinesOfLenderWithStateIn(lender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfLenderWithState)(this.subgraphUrl, lender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    countAllCreditLinesOfLenderWithStateIn_requestByLender(lender, requestByLender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getCreditLinesOfLenderWithState_requestByLender)(this.subgraphUrl, lender, requestByLender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    transformToCreditLineEmulator(data, prices, collateralPerStrategyToken) {
        return data.map((a) => {
            return new CreditLines_1.CreditLineEmulator({
                id: a.id,
                collateralShareInStrategy: new bignumber_js_1.default(a.collateralShareInStrategy),
                principal: new bignumber_js_1.default(a.principal),
                lastPrincipalUpdateTime: new bignumber_js_1.default(a.lastPrincipalUpdateTime),
                borrowRate: new bignumber_js_1.default(a.borrowRate),
                interestAccruedTillLastPrincipalUpdate: new bignumber_js_1.default(a.interestAccruedTillLastPrincipalUpdate),
                totalInterestRepaid: new bignumber_js_1.default(a.totalInterestRepaid),
                idealCollateralRatio: new bignumber_js_1.default(a.idealCollateralRatio),
                creditLineStatus: this.convertToCreditLineStatusEnum(a.status),
                borrowLimit: new bignumber_js_1.default(a.borrowLimit),
                createdAt: new bignumber_js_1.default(a.createdAt),
                borrowAsset: a.borrowAsset,
                collateralAsset: a.collateralAsset,
                autoLiquidation: new Boolean(a.autoLiquidation).valueOf(),
                lender: a.lender,
                borrower: a.borrower,
                requestByLender: new Boolean(a.requestByLender).valueOf(),
                borrowAssetStrategy: a.borrowAssetStrategy,
                collateralAssetStrategy: a.collateralAssetStrategy,
            }, {
                collateralPerStrategyToken: new bignumber_js_1.default(collateralPerStrategyToken[a.collateralAssetStrategy][a.collateralAsset]),
                ratioOfPrices: new bignumber_js_1.default(prices[a.collateralAsset]).dividedBy(prices[a.borrowAsset]),
                ratioOfPricesDecimals: 0,
            }, { liquidatorRewardFraction: new bignumber_js_1.default(10).pow(16) });
        });
    }
}
exports.CreditLineCalls = CreditLineCalls;
//# sourceMappingURL=CreditLine.js.map