import { Signer } from '@ethersproject/abstract-signer';
import { TokenManager } from '../tokenManager';
import { SublimeConfig } from '../types/sublimeConfig';
import { CreditLineCalls } from './CreditLine';
import { PooledCreditLineDetail, CreditLineStatus, LenderContributionToPooledCreditLines, PooledCreditLineOperation, LenderPoolDetail, LenderPerPool } from '../types/Types';
import BigNumber from 'bignumber.js';
import { PooledCreditLineEmulator } from '../emulator/PooledCreditLines';
export declare class PooledCreditLineCalls extends CreditLineCalls {
    constructor(url: string, signer: Signer, tokenManager: TokenManager, config: SublimeConfig);
    /**
     * @description Count all the pooled credit lines
     * @returns
     */
    countAllPooledCreditLines(): Promise<number>;
    /**
     * @description Count all pooled credit lines of a borrower
     * @param borrower
     * @returns
     */
    countAllPooledCreditLinesOfBorrower(borrower: string): Promise<number>;
    /**
     * @description Count all Pooled Credit Lines of a lender
     * @param lender
     * @returns
     */
    countAllPooledCreditLinesOfLender(lender: string): Promise<number>;
    /**
     * @description Returns pooled credit ordered by latest created
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLines(count?: number, skip?: number): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Returns all PCL which have state
     * @param state
     * @param count
     * @param skip
     * @returns
     */
    getAllPoolCreditLinesWithStateIn(state: CreditLineStatus[], count?: number, skip?: number): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Returns all PCL which do not have state
     * @param state
     * @param count
     * @param skip
     * @returns
     */
    getAllPoolCreditLinesWithStateNotIn(state: CreditLineStatus[], count?: number, skip?: number): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Returns specific PCL details
     * @param id
     * @returns
     */
    getPooledCreditLineById(id: number | string): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Returns all PCL of borrower
     * @param address
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfBorrower(address: string, count?: number, skip?: number): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Returns all the PCL of borrower with state in
     * @param borrower
     * @param status
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfBorrowerWithStateIn(borrower: string, status: CreditLineStatus[], count?: number, skip?: number): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Returns all the PCL of borrower with state not in
     * @param borrower
     * @param status
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfBorrowerWithStateNotIn(borrower: string, status: CreditLineStatus[], count?: number, skip?: number): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Returns all the PCL which a lender can lend to
     * @param lender
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesLenderCanLendTo(lender: string, count?: number, skip?: number): Promise<PooledCreditLineDetail[]>;
    /**
     * @description Get All pooled credit lines of lender
     * @param lender
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfLender(lender: string, count?: number, skip?: number): Promise<[PooledCreditLineDetail[], LenderContributionToPooledCreditLines[]]>;
    getSpecificPooledCreditLineOfLender(lender: string, id: string): Promise<[PooledCreditLineDetail | undefined, LenderContributionToPooledCreditLines | undefined]>;
    /**
     * @deprecated Will be removed in future
     * @param user
     * @returns
     */
    getAllPooledCreditLinesOfUser(user: string): Promise<PooledCreditLineDetail[]>;
    getAllLendersPerPool(pclIds: string[]): Promise<any>;
    /**
     * @description Fetch PCL timeline
     * @param pooledCreditLineId
     * @param count
     * @param skip
     * @returns
     */
    getPooledCreditLineTimeline(pooledCreditLineId: string, count?: number, skip?: number): Promise<PooledCreditLineOperation[]>;
    private transformToPooledCreditLineOperation;
    /**
     * @description Get Lenders Per Pool
     * @todo Add pagination to LendersPerPool query
     * @param id
     * @returns
     */
    getLendersPerPool(id: string): Promise<LenderPoolDetail>;
    private transformToLenderPoolDetail;
    private transformToLenderPerPoolDetail;
    /**
     * @description Count all PCL with states in
     * @param state
     * @returns
     */
    countAllPooledCreditLinesWithStates(state: CreditLineStatus[]): Promise<number>;
    /**
     * @description Count all PCL with states not in
     * @param state
     * @returns
     */
    countAllPooledCreditLinesWithStatesNotIn(state: CreditLineStatus[]): Promise<number>;
    /**
     * @description Count all PCL of borrower with state in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfBorrowerWithState(borrower: string, state: CreditLineStatus[]): Promise<number>;
    /**
     * @description Count all PCL of borrower with state not in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfBorrowerWithStateNotIn(borrower: string, state: CreditLineStatus[]): Promise<number>;
    /**
     * @description Count all PCL of lender with state in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfLenderWithState(lender: string, state: CreditLineStatus[]): Promise<number>;
    /**
     * @description Count all PCL of lender with state not in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfLenderWithStateNotIn(lender: string, state: CreditLineStatus[]): Promise<number>;
    /**
     * @description Count all Lenders of PooledCreditLine
     * @param id
     * @returns
     */
    countAllLendersOfPooledCreditLine(id: number): Promise<number>;
    protected transformToPooledCreditLineEmulator(pclData: any[], lenderPoolData: any[], lendersPerPool: LenderPerPool[][], prices: Record<string, BigNumber>, collateralPerStrategyToken: Record<string, Record<string, BigNumber>>): PooledCreditLineEmulator[];
    private transformToPooledCreditLine;
    private transformToLenderContributionToPooledCreditLines;
    private convertToPooledCreditLineStatusEnum;
}
