"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SavingsAccountCalls = void 0;
const Pools_1 = require("./Pools");
const queries_1 = require("../queries");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const SCALING_FACTOR = new bignumber_js_1.default(10).pow(18);
class SavingsAccountCalls extends Pools_1.PoolCalls {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    getAllowances(address) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!address) {
                address = yield this.signer.getAddress();
            }
            const result = yield (0, queries_1.getAllowances)(this.subgraphUrl, yield this.signer.getAddress(), this.sublimeAddresses.creditLineContractAddress);
            return this.transformToAllowances(result);
        });
    }
    transformToAllowances(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = data.map((a) => a.token).filter((value, index, array) => array.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
            }
            const prices = {};
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            return data.map((a) => {
                return {
                    amount: { value: a.amount, decimals: this.tokenManager.getTokenDecimals(a.token) },
                    from: a.from,
                    to: a.to,
                    token: {
                        address: a.token,
                        name: this.tokenManager.getTokenName(a.token),
                        logo: this.tokenManager.getLogo(a.token),
                        pricePerAssetInUSD: prices[a.token],
                        decimals: this.tokenManager.getTokenDecimals(a.token),
                    },
                };
            });
        });
    }
    /**
     *
     * @param address: address of the user to query
     * @description Returns savings account overview for a user address
     */
    getSavingsAccountOverview(address) {
        return __awaiter(this, void 0, void 0, function* () {
            const balances = yield (0, queries_1.getBalances)(this.subgraphUrl, address);
            const savingsAccountUserDetails = yield this.newTransformToSavingsAccountUserDetails(address, balances);
            return savingsAccountUserDetails;
        });
    }
    newTransformToSavingsAccountUserDetails(address, data) {
        return __awaiter(this, void 0, void 0, function* () {
            // const allowances = await this.getAllowances(address);
            // console.log({allowances});
            const savingsAccountUserDetails = {
                user: address.toLowerCase(),
                totalBalance: { value: '0', decimals: 0 },
                balances: [],
            };
            const result = yield this.transformToInternalBalancePerTokenStrategy(data);
            const stackedBalancesByToken = yield this.stackInternalBalanceByToken(result);
            savingsAccountUserDetails.balances.push(...stackedBalancesByToken);
            const totalBalance = stackedBalancesByToken.reduce((total, current) => total.plus(current.balanceUSD.value.toString()), new bignumber_js_1.default(0));
            savingsAccountUserDetails.totalBalance = { value: totalBalance.toString(), decimals: 0 };
            return savingsAccountUserDetails;
        });
    }
    getAprData(internalBalances) {
        return __awaiter(this, void 0, void 0, function* () {
            const allStrategyAndTokenPairs = internalBalances.map((a) => {
                return { strategy: a.strategy.address, token: a.token.address, id: `${a.strategy.address}-${a.token.address}` };
            });
            const allIds = allStrategyAndTokenPairs.map((a) => a.id).filter((value, index, self) => self.indexOf(value) === index);
            const uniquePairs = allIds.map((a) => {
                return { strategy: a.split('-')[0], token: a.split('-')[1] };
            });
            const aprPerStrategyToken = {};
            for (let index = 0; index < uniquePairs.length; index++) {
                const element = uniquePairs[index];
                if (!aprPerStrategyToken[element.strategy]) {
                    aprPerStrategyToken[element.strategy] = {};
                }
                aprPerStrategyToken[element.strategy][element.token] = yield this.getAPR(this.yieldApi.getStrategy(element.strategy), element.token);
            }
            return aprPerStrategyToken;
        });
    }
    stackInternalBalanceByToken(internalBalances) {
        return __awaiter(this, void 0, void 0, function* () {
            const stackedBalanceByToken = [];
            const uniqueTokens = internalBalances.map((a) => a.token.address).filter((value, index, self) => self.indexOf(value) === index);
            const prices = {};
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            const aprs = yield this.getAprData(internalBalances);
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                const requiredElements = internalBalances.filter((a) => a.token.address === element);
                if (requiredElements.length > 0) {
                    const totalBalance = requiredElements.reduce((total, current) => total.plus(current.amount.value.toString()), new bignumber_js_1.default(0));
                    const totalBalanceUSD = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amount.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .div(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    const totalAprWeight = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amount.value.toString()).multipliedBy(aprs[current.strategy.address][current.token.address])), new bignumber_js_1.default(0));
                    stackedBalanceByToken.push({
                        token: requiredElements[0].token,
                        balance: { value: totalBalance.toString(), decimals: this.tokenManager.getTokenDecimals(requiredElements[0].token.address) },
                        amountAllocatedToCreditLines: { value: '0', decimals: this.tokenManager.getTokenDecimals(requiredElements[0].token.address) },
                        balanceUSD: { value: totalBalanceUSD.toString(), decimals: 0 },
                        strategyBalance: [...this.transformToSavingsAccountStrategyBalanceDisplay(requiredElements, aprs, prices)],
                        APR: totalAprWeight.dividedBy(totalBalance).dividedBy(SCALING_FACTOR).toFixed(),
                    });
                }
            }
            return stackedBalanceByToken;
        });
    }
    transformToSavingsAccountStrategyBalanceDisplay(internalBalances, aprs, prices) {
        return internalBalances.map((a) => {
            return {
                strategy: a.strategy,
                balance: a.amount,
                balanceUSD: {
                    value: new bignumber_js_1.default(a.amount.value.toString())
                        .multipliedBy(prices[a.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(a.token.address)))
                        .toString(),
                    decimals: 0,
                },
                APR: aprs[a.strategy.address][a.token.address].div(SCALING_FACTOR).toFixed(),
            };
        });
    }
    transformToInternalBalancePerTokenStrategy(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = data.map((a) => a.token).filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
            }
            const prices = {};
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            const all = data.map((a) => __awaiter(this, void 0, void 0, function* () {
                return {
                    token: {
                        address: a.token,
                        name: this.tokenManager.getTokenName(a.token),
                        logo: this.tokenManager.getLogo(a.token),
                        pricePerAssetInUSD: prices[a.token],
                        decimals: this.tokenManager.getTokenDecimals(a.token),
                    },
                    strategy: {
                        address: a.strategy.address,
                        type: this.yieldApi.getStrategy(a.strategy.address),
                        displayName: this.yieldApi.getStrategyDisplayName(a.strategy.address),
                        logo: this.yieldApi.getStrategyLogo(a.strategy.address),
                    },
                    shares: { value: a.shares, decimals: 0 },
                    amount: {
                        value: yield (yield this.getTokensForShares(this.yieldApi.getStrategy(a.strategy.address), a.token, new bignumber_js_1.default(a.shares))).toString(),
                        decimals: this.tokenManager.getTokenDecimals(a.token),
                    },
                };
            }));
            return Promise.all(all);
        });
    }
    /**
     * @param strategy
     * @@description calulcates the APR for a given strategy
     */
    getAPR(strategy, asset) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.yieldApi.getApr(strategy, asset);
        });
    }
}
exports.SavingsAccountCalls = SavingsAccountCalls;
//# sourceMappingURL=SavingsAccount.js.map