"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenManager = void 0;
const ERC20__factory_1 = require("./wrappers/factories/ERC20__factory");
const bignumber_js_1 = require("bignumber.js");
const prices_1 = require("./queries/prices");
const tokenMapping_1 = require("./config/tokenMapping");
const chainlinkMapping_1 = require("./config/chainlinkMapping");
const wrappers_1 = require("./wrappers");
/**
 * @class TokenManager
 * @description An token manager stores, retrieves and caches the data like token name, decimals, logos and prices.
 */
class TokenManager {
    constructor(signer, priceSubgraphUrl) {
        /**
         * @description caches number of decimals for a token address
         */
        this.decimals = {};
        /**
         * @description caches name of token
         */
        this.names = {};
        /**
         * @description cache the price of token, Updates every 1 min
         */
        this.prices = {};
        /**
         * @description stores when last time when the token price was updated
         */
        this.priceLastUpdatedAt = {};
        /**
         * @description caches the token logo
         */
        this.logos = {};
        /**
         * @description used for mapping tokens logos of testnet
         */
        this.addressMapper = {};
        /**
         * @description Base path for logos
         */
        this.logoUrlTemplate = 'https://tokens.1inch.io/ADDRESS.png';
        /**
         * @description interval at which prices are refreshed in the SDK
         */
        this.priceRefreshInterval = 60000;
        this.chainlinkFeedsAddress = {};
        this.signer = signer;
        this.priceSubgraphUrl = priceSubgraphUrl;
        for (let index = 0; index < tokenMapping_1.tokenData.length; index++) {
            const element = tokenMapping_1.tokenData[index];
            this.addressMapper[element.address.toLowerCase()] = element.iconMapping.toLowerCase();
            this.decimals[element.address.toLowerCase()] = element.decimals;
            this.names[element.address.toLowerCase()] = element.symbol;
        }
        for (let index = 0; index < chainlinkMapping_1.feedData.length; index++) {
            const element = chainlinkMapping_1.feedData[index];
            this.chainlinkFeedsAddress[element.tokenAddress] = element.feedAddress;
        }
    }
    /**
     * @description Updates the token meta data
     * @param tokenAddress
     */
    updateAll(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.updateLogo(tokenAddress);
            yield this.updatePricePerAsset(tokenAddress);
            yield this.updateTokenDecimals(tokenAddress);
            yield this.updateTokenName(tokenAddress);
            return;
        });
    }
    /**
     * @description Updated the token decimals
     * @param tokenAddress
     * @returns
     */
    updateTokenDecimals(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            tokenAddress = tokenAddress.toLowerCase();
            if (tokenAddress in this.decimals) {
                return;
            }
            else {
                const token = yield new ERC20__factory_1.ERC20__factory(this.signer).attach(tokenAddress);
                this.decimals[tokenAddress] = yield token.decimals();
                return;
            }
        });
    }
    /**
     * @description Updates token name
     * @param tokenAddress
     * @returns
     */
    updateTokenName(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            tokenAddress = tokenAddress.toLowerCase();
            if (tokenAddress in this.names) {
                return;
            }
            else {
                const token = yield new ERC20__factory_1.ERC20__factory(this.signer).attach(tokenAddress);
                this.names[tokenAddress] = yield token.name();
                return;
            }
        });
    }
    /**
     * @description Updates token price
     * @param tokenAddress
     * @returns
     */
    updatePricePerAsset(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            // tokenAddress = tokenAddress.toLowerCase();
            // const now = new Date().valueOf();
            // if (!(tokenAddress in this.prices) || now > this.priceRefreshInterval + this.priceLastUpdatedAt[tokenAddress]) {
            //   let mappedAddress = tokenAddress;
            //   if (this.addressMapper[tokenAddress]) {
            //     mappedAddress = this.addressMapper[tokenAddress];
            //   }
            //   this.prices[tokenAddress] = await getUniswapPrice(this.priceSubgraphUrl, mappedAddress);
            //   this.priceLastUpdatedAt[tokenAddress] = new Date().valueOf();
            // }
            yield this._updatePricePerAsset(tokenAddress);
            return;
        });
    }
    _updatePricePerAsset(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            tokenAddress = tokenAddress.toLowerCase();
            const now = new Date().valueOf();
            if (this.chainlinkFeedsAddress[tokenAddress]) {
                if (!(tokenAddress in this.prices) || now > this.priceRefreshInterval + this.priceLastUpdatedAt[tokenAddress]) {
                    const feedAddress = this.chainlinkFeedsAddress[tokenAddress];
                    const aggregator = wrappers_1.AggregatorV3Interface__factory.connect(feedAddress, this.signer);
                    const feedData = yield aggregator.callStatic.latestRoundData();
                    const decimals = (yield aggregator.callStatic.decimals()).toString();
                    const price = feedData.answer.toString();
                    this.prices[tokenAddress] = new bignumber_js_1.BigNumber(price).dividedBy(new bignumber_js_1.BigNumber(10).pow(decimals));
                    this.priceLastUpdatedAt[tokenAddress] = new Date().valueOf();
                }
                return;
            }
            else {
                if (!(tokenAddress in this.prices) || now > this.priceRefreshInterval + this.priceLastUpdatedAt[tokenAddress]) {
                    let mappedAddress = tokenAddress;
                    if (this.addressMapper[tokenAddress]) {
                        mappedAddress = this.addressMapper[tokenAddress];
                    }
                    this.prices[tokenAddress] = yield (0, prices_1.getUniswapPrice)(this.priceSubgraphUrl, mappedAddress);
                    this.priceLastUpdatedAt[tokenAddress] = new Date().valueOf();
                }
                return;
            }
        });
    }
    /**
     * @description Updates token Logo
     * @param tokenAddress
     * @returns
     */
    updateLogo(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            tokenAddress = tokenAddress.toLowerCase();
            if (tokenAddress in this.logos) {
                return;
            }
            else {
                let mappedAddress = tokenAddress;
                if (this.addressMapper[tokenAddress]) {
                    mappedAddress = this.addressMapper[tokenAddress];
                }
                this.logos[tokenAddress] = this.logoUrlTemplate.replace('ADDRESS', mappedAddress);
                return;
            }
        });
    }
    /**
     * @param tokenAddress
     * @returns number of token decimals
     */
    getTokenDecimals(tokenAddress) {
        tokenAddress = tokenAddress.toLowerCase();
        if (tokenAddress in this.decimals) {
            return this.decimals[tokenAddress];
        }
        else {
            throw new Error('Error in class TokenManager');
        }
    }
    /**
     * @param tokenAddress
     * @returns token name
     */
    getTokenName(tokenAddress) {
        tokenAddress = tokenAddress.toLowerCase();
        if (tokenAddress in this.names) {
            return this.names[tokenAddress];
        }
        else {
            throw new Error('Error in class TokenManager');
        }
    }
    /**
     * @param tokenAddress
     * @returns price of asset
     */
    getPricePerAsset(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            tokenAddress = tokenAddress.toLowerCase();
            yield this.updatePricePerAsset(tokenAddress);
            if (tokenAddress in this.prices) {
                return this.prices[tokenAddress];
            }
            else {
                throw new Error('Error in class TokenManager');
            }
        });
    }
    /**
     * @param tokenAddress
     * @returns return url which renders the logo/pic of the token
     */
    getLogo(tokenAddress) {
        tokenAddress = tokenAddress.toLowerCase();
        if (tokenAddress in this.logos) {
            return this.logos[tokenAddress];
        }
        else {
            throw new Error('Error in class TokenManager');
        }
    }
    /**
     * @param tokenAddress
     * @returns
     */
    getAssetMeta(tokenAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.updateAll(tokenAddress);
            tokenAddress = tokenAddress.toLowerCase();
            return {
                name: this.getTokenName(tokenAddress),
                address: tokenAddress,
                logo: this.getLogo(tokenAddress),
                pricePerAssetInUSD: (yield this.getPricePerAsset(tokenAddress)).toString(),
                decimals: this.getTokenDecimals(tokenAddress),
            };
        });
    }
}
exports.TokenManager = TokenManager;
//# sourceMappingURL=tokenManager.js.map